#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock
from webtest.app import AppError

from magnum.common import clients
from magnum.tests.unit.api import utils as apiutils
from magnum.tests.unit.common.policies import base
from magnum.tests.unit.objects import utils as obj_utils


class TestQuotaPolicy(base.PolicyFunctionalTest):
    def setUp(self):
        super(TestQuotaPolicy, self).setUp()

    def test_get_all_no_permission(self):
        exc = self.assertRaises(
            AppError, self.get_json, '/quotas',
            headers=self.reader_headers)
        self.assertIn("403 Forbidden", str(exc))

    def test_get_no_permission(self):
        quota = obj_utils.create_test_quota(self.context)
        exc = self.assertRaises(
            AppError,
            self.get_json,
            f"/quotas/{quota['project_id']}/{quota['resource']}",
            headers=self.member_headers)
        self.assertIn("403 Forbidden", str(exc))

    @mock.patch.object(clients.OpenStackClients, 'keystone')
    def test_create_no_permission(self, mock_keystone):
        exc = self.assertRaises(
            AppError, self.post_json,
            '/quotas', apiutils.quota_post_data(),
            headers=self.reader_headers)
        self.assertIn("403 Forbidden", str(exc))

    @mock.patch.object(clients.OpenStackClients, 'keystone')
    def test_update_no_permission(self, mock_keystone):
        with mock.patch("magnum.common.policy.enforce"):
            quota_dict = apiutils.quota_post_data(hard_limit=5)
            self.post_json('/quotas', quota_dict)
        quota_dict['hard_limit'] = 20
        exc = self.assertRaises(
            AppError, self.patch_json, '/quotas', quota_dict,
            headers=self.reader_headers)
        self.assertIn("403 Forbidden", str(exc))

    @mock.patch.object(clients.OpenStackClients, 'keystone')
    def test_delete_no_permission(self, mock_keystone):
        with mock.patch("magnum.common.policy.enforce"):
            quota_dict = apiutils.quota_post_data()
            response = self.post_json('/quotas', quota_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(201, response.status_int)

        project_id = quota_dict['project_id']
        resource = quota_dict['resource']
        # delete quota
        exc = self.assertRaises(
            AppError, self.delete, f"/quotas/{project_id}/{resource}",
            headers=self.reader_headers)
        self.assertIn("403 Forbidden", str(exc))



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from unittest import mock

from webtest.app import AppError

from magnum.tests.unit.api import base as api_base
from magnum.tests.unit.objects import utils as obj_utils


class TestStatsController(api_base.FunctionalTest):

    def setUp(self):
        self.base_headers = {
            "X-Roles": "reader",
            "OpenStack-API-Version": "container-infra 1.4"
        }
        self.base_admin_headers = {
            "X-Roles": "admin",
            "OpenStack-API-Version": "container-infra 1.4"
        }
        super(TestStatsController, self).setUp()
        obj_utils.create_test_cluster_template(self.context)

    def test_empty(self):
        response = self.get_json('/stats', headers=self.base_headers)
        expected = {u'clusters': 0, u'nodes': 0}
        self.assertEqual(expected, response)

    @mock.patch("magnum.common.policy.enforce")
    @mock.patch("magnum.common.context.make_context")
    def test_admin_get_all_stats(self, mock_context, mock_policy):
        obj_utils.create_test_cluster(self.context,
                                      project_id=123,
                                      uuid='uuid1')
        obj_utils.create_test_cluster(self.context,
                                      project_id=234,
                                      uuid='uuid2')
        response = self.get_json('/stats', headers=self.base_admin_headers)
        expected = {u'clusters': 2, u'nodes': 12}
        self.assertEqual(expected, response)

    @mock.patch("magnum.common.policy.enforce")
    @mock.patch("magnum.common.context.make_context")
    def test_admin_get_tenant_stats(self, mock_context, mock_policy):
        obj_utils.create_test_cluster(self.context,
                                      project_id=123,
                                      uuid='uuid1')
        obj_utils.create_test_cluster(self.context,
                                      project_id=234,
                                      uuid='uuid2')
        self.context.is_admin = True
        response = self.get_json('/stats?project_id=234',
                                 headers=self.base_admin_headers)
        expected = {u'clusters': 1, u'nodes': 6}
        self.assertEqual(expected, response)

    @mock.patch("magnum.common.policy.enforce")
    @mock.patch("magnum.common.context.make_context")
    def test_admin_get_invalid_tenant_stats(self, mock_context, mock_policy):
        obj_utils.create_test_cluster(self.context,
                                      project_id=123,
                                      uuid='uuid1')
        obj_utils.create_test_cluster(self.context,
                                      project_id=234,
                                      uuid='uuid2')
        self.context.is_admin = True
        response = self.get_json('/stats?project_id=34',
                                 headers=self.base_admin_headers)
        expected = {u'clusters': 0, u'nodes': 0}
        self.assertEqual(expected, response)

    def test_get_self_stats(self):
        obj_utils.create_test_cluster(self.context,
                                      project_id=123,
                                      uuid='uuid1')
        obj_utils.create_test_cluster(self.context,
                                      project_id=234,
                                      uuid='uuid2',
                                      node_count=5,
                                      master_count=1)
        headers = self.base_headers.copy()
        headers['X-Project-Id'] = '234'
        response = self.get_json('/stats',
                                 headers=headers)
        expected = {u'clusters': 1, u'nodes': 6}
        self.assertEqual(expected, response)

    def test_get_self_stats_without_param(self):
        obj_utils.create_test_cluster(self.context,
                                      project_id=123,
                                      uuid='uuid1')
        obj_utils.create_test_cluster(self.context,
                                      project_id=234,
                                      uuid='uuid2',
                                      node_count=5,
                                      master_count=1)
        headers = self.base_headers.copy()
        headers['X-Project-Id'] = '234'
        response = self.get_json('/stats',
                                 headers=headers)
        expected = {u'clusters': 1, u'nodes': 6}
        self.assertEqual(expected, response)

    def test_get_some_other_user_stats(self):
        obj_utils.create_test_cluster(self.context,
                                      project_id=123,
                                      uuid='uuid1')
        obj_utils.create_test_cluster(self.context,
                                      project_id=234,
                                      uuid='uuid2',
                                      node_count=5)
        headers = self.base_headers.copy()
        headers['X-Project-Id'] = '234'
        self.assertRaises(AppError,
                          self.get_json,
                          '/stats?project_id=123',
                          headers=headers)

    def test_get_invalid_type_stats(self):
        obj_utils.create_test_cluster(self.context,
                                      project_id=123,
                                      uuid='uuid1')
        self.assertRaises(AppError,
                          self.get_json,
                          '/stats?project_id=123&type=invalid',
                          headers=self.base_headers)



# Copyright (c) 2018 European Organization for Nuclear Research.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""add nodegoup table

Revision ID: ac92cbae311c
Revises: cbbc65a86986
Create Date: 2018-09-20 15:26:00.869885

"""

# revision identifiers, used by Alembic.
revision = 'ac92cbae311c'
down_revision = '87e62e3c7abc'

from alembic import op  # noqa: E402

import sqlalchemy as sa  # noqa: E402

from sqlalchemy.types import String  # noqa: E402

from magnum.db.sqlalchemy import models  # noqa: E402


def upgrade():
    op.create_table(
        'nodegroup',
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('uuid', String(length=36), nullable=False),
        sa.Column('name', String(length=255), nullable=False),
        sa.Column('cluster_id', String(length=255), nullable=False),
        sa.Column('project_id', String(length=255), nullable=False),
        sa.Column('docker_volume_size', sa.Integer(), nullable=True),
        sa.Column('labels', models.JSONEncodedDict, nullable=True),
        sa.Column('flavor_id', String(length=255), nullable=True),
        sa.Column('image_id', String(length=255), nullable=True),
        sa.Column('node_addresses', models.JSONEncodedList(), nullable=True),
        sa.Column('node_count', sa.Integer(), nullable=True),
        sa.Column('max_node_count', sa.Integer(), nullable=True),
        sa.Column('min_node_count', sa.Integer(), nullable=True),
        sa.Column('role', String(length=255), nullable=True),
        sa.Column('is_default',
                  sa.Boolean(create_constraint=False),
                  default=False),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('uuid', name='uniq_nodegroup0uuid'),
        sa.UniqueConstraint('cluster_id', 'name',
                            name='uniq_nodegroup0cluster_id0name'),
    )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_config import fixture

from keystoneauth1 import exceptions as ka_exception
from keystoneauth1 import identity as ka_identity
import keystoneclient.exceptions as kc_exception

from magnum.common import exception
from magnum.common import keystone
import magnum.conf
from magnum.conf import keystone as ksconf
from magnum.tests import base
from magnum.tests import utils

CONF = magnum.conf.CONF


@mock.patch('keystoneclient.v3.client.Client')
class KeystoneClientTest(base.TestCase):

    def setUp(self):
        super(KeystoneClientTest, self).setUp()
        dummy_url = 'http://server.test:5000/v3'

        self.ctx = utils.dummy_context()
        self.ctx.auth_url = dummy_url
        self.ctx.auth_token = 'abcd1234'

        plugin = keystone.ka_loading.get_plugin_loader('password')
        opts = keystone.ka_loading.get_auth_plugin_conf_options(plugin)
        cfg_fixture = self.useFixture(fixture.Config())
        cfg_fixture.register_opts(opts, group=ksconf.CFG_GROUP)
        self.config(auth_type='password',
                    auth_url=dummy_url,
                    username='fake_user',
                    password='fake_pass',
                    project_name='fake_project',
                    group=ksconf.CFG_GROUP)

        self.config(auth_uri=dummy_url,
                    admin_user='magnum',
                    admin_password='varybadpass',
                    admin_tenant_name='service',
                    group=ksconf.CFG_LEGACY_GROUP)

        # Disable global mocking for trustee_domain_id
        self.stop_global(
            'magnum.common.keystone.KeystoneClientV3.trustee_domain_id')

    def tearDown(self):
        # Re-enable global mocking for trustee_domain_id. We need this because
        # mock blows up when trying to stop an already stopped patch (which it
        # will do due to the addCleanup() in base.TestCase).
        self.start_global(
            'magnum.common.keystone.KeystoneClientV3.trustee_domain_id')

        super(KeystoneClientTest, self).tearDown()

    def test_client_with_password(self, mock_ks):
        self.ctx.is_admin = True
        self.ctx.auth_token_info = None
        self.ctx.auth_token = None
        self.ctx.trust_id = None
        ks_client = keystone.KeystoneClientV3(self.ctx)
        ks_client.client
        session = ks_client.session
        auth_plugin = session.auth
        mock_ks.assert_called_once_with(session=session, trust_id=None)
        self.assertIsInstance(auth_plugin, ka_identity.Password)

    @mock.patch('magnum.common.keystone.ka_loading')
    @mock.patch('magnum.common.keystone.ka_v3')
    def test_client_with_password_legacy(self, mock_v3, mock_loading, mock_ks):
        self.ctx.is_admin = True
        self.ctx.auth_token_info = None
        self.ctx.auth_token = None
        self.ctx.trust_id = None
        mock_loading.load_auth_from_conf_options.side_effect = \
            ka_exception.MissingRequiredOptions(mock.MagicMock())
        ks_client = keystone.KeystoneClientV3(self.ctx)
        ks_client.client
        session = ks_client.session
        self.assertWarnsRegex(Warning,
                              '[keystone_authtoken] section is deprecated')
        mock_v3.Password.assert_called_once_with(
            auth_url='http://server.test:5000/v3', password='varybadpass',
            project_domain_id='default', project_name='service',
            user_domain_id='default', username='magnum')
        mock_ks.assert_called_once_with(session=session, trust_id=None)

    @mock.patch('magnum.common.keystone.ka_access')
    def test_client_with_access_info(self, mock_access, mock_ks):
        self.ctx.auth_token_info = mock.MagicMock()
        ks_client = keystone.KeystoneClientV3(self.ctx)
        ks_client.client
        session = ks_client.session
        auth_plugin = session.auth
        mock_access.create.assert_called_once_with(body=mock.ANY,
                                                   auth_token='abcd1234')
        mock_ks.assert_called_once_with(session=session, trust_id=None)
        self.assertIsInstance(auth_plugin, ka_identity.access.AccessInfoPlugin)

    @mock.patch('magnum.common.keystone.ka_v3')
    def test_client_with_token(self, mock_v3, mock_ks):
        ks_client = keystone.KeystoneClientV3(self.ctx)
        ks_client.client
        session = ks_client.session
        mock_v3.Token.assert_called_once_with(
            auth_url='http://server.test:5000/v3', token='abcd1234')
        mock_ks.assert_called_once_with(session=session, trust_id=None)

    def test_client_with_no_credentials(self, mock_ks):
        self.ctx.auth_token = None
        ks_client = keystone.KeystoneClientV3(self.ctx)
        self.assertRaises(exception.AuthorizationFailure,
                          ks_client._get_auth)
        mock_ks.assert_not_called()

    def test_delete_trust(self, mock_ks):
        mock_ks.return_value.trusts.delete.return_value = None
        ks_client = keystone.KeystoneClientV3(self.ctx)
        cluster = mock.MagicMock()
        cluster.trust_id = 'atrust123'
        self.assertIsNone(ks_client.delete_trust(self.ctx, cluster))
        mock_ks.return_value.trusts.delete.assert_called_once_with('atrust123')

    def test_delete_trust_not_found(self, mock_ks):
        mock_delete = mock_ks.return_value.trusts.delete
        mock_delete.side_effect = kc_exception.NotFound()
        ks_client = keystone.KeystoneClientV3(self.ctx)
        cluster = mock.MagicMock()
        cluster.trust_id = 'atrust123'
        self.assertIsNone(ks_client.delete_trust(self.ctx, cluster))

    @mock.patch('keystoneauth1.session.Session')
    def test_create_trust_with_all_roles(self, mock_session, mock_ks):
        mock_session.return_value.get_user_id.return_value = '123456'
        mock_session.return_value.get_project_id.return_value = '654321'

        self.ctx.roles = ['role1', 'role2']
        ks_client = keystone.KeystoneClientV3(self.ctx)

        ks_client.create_trust(trustee_user='888888')

        mock_ks.return_value.trusts.create.assert_called_once_with(
            allow_redelegation=False,
            trustor_user='123456', project='654321',
            trustee_user='888888', role_names=['role1', 'role2'],
            impersonation=True)

    @mock.patch('keystoneauth1.session.Session')
    def test_create_trust_with_limit_roles(self, mock_session, mock_ks):
        mock_session.return_value.get_user_id.return_value = '123456'
        mock_session.return_value.get_project_id.return_value = '654321'

        self.ctx.roles = ['role1', 'role2']
        ks_client = keystone.KeystoneClientV3(self.ctx)

        CONF.set_override('roles', ['role3'], group='trust')
        ks_client.create_trust(trustee_user='888888')

        mock_ks.return_value.trusts.create.assert_called_once_with(
            allow_redelegation=False,
            trustor_user='123456', project='654321',
            trustee_user='888888', role_names=['role3'],
            impersonation=True)

    @mock.patch('magnum.common.keystone.KeystoneClientV3.trustee_domain_id')
    def test_create_trustee(self, mock_tdi, mock_ks):
        expected_username = '_username'
        expected_password = '_password'
        expected_domain = '_expected_trustee_domain_id'
        mock_tdi.__get__ = mock.MagicMock(return_value=expected_domain)

        ks_client = keystone.KeystoneClientV3(self.ctx)
        ks_client.create_trustee(
            username=expected_username,
            password=expected_password,
        )
        mock_ks.return_value.users.create.assert_called_once_with(
            name=expected_username,
            password=expected_password,
            domain=expected_domain,
        )

    @mock.patch('magnum.common.keystone.KeystoneClientV3.domain_admin_auth')
    @mock.patch('magnum.common.keystone.KeystoneClientV3.domain_admin_session')
    def test_trustee_domain_id(self, mock_session, mock_auth, mock_ks):
        expected_domain_id = '_expected_domain_id'
        _mock_session = mock.MagicMock()
        mock_session.__get__ = mock.MagicMock(return_value=_mock_session)
        _mock_auth = mock.MagicMock()
        mock_auth.__get__ = mock.MagicMock(return_value=_mock_auth)
        mock_access = mock.MagicMock()
        mock_access.domain_id = expected_domain_id
        _mock_auth.get_access.return_value = mock_access

        ks_client = keystone.KeystoneClientV3(self.ctx)
        self.assertEqual(expected_domain_id, ks_client.trustee_domain_id)

        _mock_auth.get_access.assert_called_once_with(
            _mock_session
        )

    def test_get_validate_region_name(self, mock_ks):
        key = 'region_name'
        val = 'RegionOne'
        CONF.set_override(key, val, 'cinder_client')
        mock_region = mock.MagicMock()
        mock_region.id = 'RegionOne'
        mock_ks.return_value.regions.list.return_value = [mock_region]
        ks_client = keystone.KeystoneClientV3(self.ctx)
        region_name = ks_client.get_validate_region_name(val)
        self.assertEqual('RegionOne', region_name)

    def test_get_validate_region_name_not_found(self, mock_ks):
        key = 'region_name'
        val = 'region123'
        CONF.set_override(key, val, 'cinder_client')
        ks_client = keystone.KeystoneClientV3(self.ctx)
        self.assertRaises(exception.InvalidParameterValue,
                          ks_client.get_validate_region_name, val)

    def test_get_validate_region_name_is_None(self, mock_ks):
        key = 'region_name'
        val = None
        CONF.set_override(key, val, 'cinder_client')
        ks_client = keystone.KeystoneClientV3(self.ctx)
        self.assertRaises(exception.InvalidParameterValue,
                          ks_client.get_validate_region_name, val)



# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_versionedobjects.tests import test_fields

from magnum.objects import fields


class TestClusterStatus(test_fields.TestField):
    def setUp(self):
        super(TestClusterStatus, self).setUp()
        self.field = fields.ClusterStatusField()
        self.coerce_good_values = [('CREATE_IN_PROGRESS',
                                    'CREATE_IN_PROGRESS'),
                                   ('CREATE_FAILED', 'CREATE_FAILED'),
                                   ('CREATE_COMPLETE', 'CREATE_COMPLETE'),
                                   ('UPDATE_IN_PROGRESS',
                                    'UPDATE_IN_PROGRESS'),
                                   ('UPDATE_FAILED', 'UPDATE_FAILED'),
                                   ('UPDATE_COMPLETE', 'UPDATE_COMPLETE'),
                                   ('DELETE_IN_PROGRESS',
                                    'DELETE_IN_PROGRESS'),
                                   ('DELETE_FAILED', 'DELETE_FAILED'),
                                   ('RESUME_COMPLETE', 'RESUME_COMPLETE'),
                                   ('RESTORE_COMPLETE', 'RESTORE_COMPLETE'),
                                   ('ROLLBACK_COMPLETE', 'ROLLBACK_COMPLETE'),
                                   ('SNAPSHOT_COMPLETE', 'SNAPSHOT_COMPLETE'),
                                   ('CHECK_COMPLETE', 'CHECK_COMPLETE'),
                                   ('ADOPT_COMPLETE', 'ADOPT_COMPLETE')]
        self.coerce_bad_values = ['DELETE_STOPPED']
        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'UPDATE_FAILED'",
                         self.field.stringify('UPDATE_FAILED'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'DELETE_STOPPED')


class TestClusterHealthStatus(test_fields.TestField):
    def setUp(self):
        super(TestClusterHealthStatus, self).setUp()
        self.field = fields.ClusterHealthStatusField()
        self.coerce_good_values = [('HEALTHY', 'HEALTHY'),
                                   ('UNHEALTHY', 'UNHEALTHY')]
        self.coerce_bad_values = ['FAKE']

        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'UNHEALTHY'",
                         self.field.stringify('UNHEALTHY'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'FAKE')


class TestContainerStatus(test_fields.TestField):
    def setUp(self):
        super(TestContainerStatus, self).setUp()
        self.field = fields.ContainerStatusField()
        self.coerce_good_values = [('Error', 'Error'), ('Running', 'Running'),
                                   ('Stopped', 'Stopped'),
                                   ('Paused', 'Paused'),
                                   ('Unknown', 'Unknown'), ]
        self.coerce_bad_values = ['DELETED']

        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'Stopped'",
                         self.field.stringify('Stopped'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'DELETED')


class TestClusterType(test_fields.TestField):
    def setUp(self):
        super(TestClusterType, self).setUp()
        self.field = fields.ClusterTypeField()
        self.coerce_good_values = [('kubernetes', 'kubernetes')]
        self.coerce_bad_values = ['invalid']

        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'kubernetes'",
                         self.field.stringify('kubernetes'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'invalid')


class TestMagnumServiceBinary(test_fields.TestField):
    def setUp(self):
        super(TestMagnumServiceBinary, self).setUp()
        self.field = fields.MagnumServiceBinaryField()
        self.coerce_good_values = [('magnum-conductor', 'magnum-conductor')]
        self.coerce_bad_values = ['invalid']

        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'magnum-conductor'",
                         self.field.stringify('magnum-conductor'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'invalid')


class TestServerType(test_fields.TestField):
    def setUp(self):
        super(TestServerType, self).setUp()
        self.field = fields.ServerTypeField()
        self.coerce_good_values = [('vm', 'vm'),
                                   ('bm', 'bm'), ]
        self.coerce_bad_values = ['invalid']

        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'vm'",
                         self.field.stringify('vm'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'invalid')



# Copyright 2015 Huawei Technologies Co.,LTD.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import importlib
from unittest import mock

from magnum.api import validation as v
from magnum.common import exception
import magnum.conf
from magnum import objects
from magnum.tests import base
from magnum.tests.unit.objects import utils as obj_utils

CONF = magnum.conf.CONF


class TestValidation(base.BaseTestCase):

    def _test_enforce_cluster_type_supported(
            self, mock_cluster_template_get, mock_cluster_get_by_uuid,
            mock_pecan_request, cluster_type, assert_raised=False):

        @v.enforce_cluster_type_supported()
        def test(self, cluster):
            pass

        server_type, cluster_distro, coe = cluster_type
        cluster_template = obj_utils.get_test_cluster_template(
            mock_pecan_request.context, uuid='cluster_template_id',
            coe=coe, cluster_distro=cluster_distro, server_type=server_type)
        mock_cluster_template_get.return_value = cluster_template

        cluster = mock.MagicMock()
        cluster.cluster_template_id = 'cluster_template_id'
        cluster.cluster_template = cluster_template
        mock_cluster_get_by_uuid.return_value = cluster

        if assert_raised:
            return self.assertRaises(
                exception.ClusterTypeNotSupported, test, self, cluster)
        else:
            self.assertIsNone(test(self, cluster))

    @mock.patch('pecan.request')
    @mock.patch('magnum.objects.Cluster.get_by_uuid')
    @mock.patch('magnum.objects.ClusterTemplate.get')
    def test_enforce_cluster_type_supported(
            self, mock_cluster_template_get, mock_cluster_get_by_uuid,
            mock_pecan_request):

        cluster_type = ('vm', 'fedora-coreos', 'kubernetes')
        self._test_enforce_cluster_type_supported(
            mock_cluster_template_get, mock_cluster_get_by_uuid,
            mock_pecan_request, cluster_type)

    @mock.patch('pecan.request')
    @mock.patch('magnum.objects.Cluster.get_by_uuid')
    @mock.patch('magnum.objects.ClusterTemplate.get')
    def test_enforce_cluster_type_not_supported(
            self, mock_cluster_template_get, mock_cluster_get_by_uuid,
            mock_pecan_request):

        cluster_type = ('vm', 'foo', 'kubernetes')
        exc = self._test_enforce_cluster_type_supported(
            mock_cluster_template_get, mock_cluster_get_by_uuid,
            mock_pecan_request, cluster_type, assert_raised=True)
        self.assertEqual('Cluster type (vm, foo, kubernetes) not supported.',
                         exc.message)

    def _test_enforce_network_driver_types_create(
        self,
        network_driver_type,
        validator_allowed_network_drivers=None,
        validator_default_network_driver=None,
        coe="kubernetes",
        assert_raised=False,
    ):
        @v.enforce_network_driver_types_create()
        def test(self, cluster_template):
            pass

        cluster_template = mock.MagicMock()
        cluster_template.name = 'test_cluster_template'
        cluster_template.network_driver = network_driver_type
        cluster_template.coe = coe

        # NOTE(dalees): Patch the validator class variables directly, so the
        #               changes are removed after the test.
        with mock.patch.multiple(
            v.K8sValidator,
            supported_network_drivers=["flannel", "type1", "type2"],
            allowed_network_drivers=validator_allowed_network_drivers
            or v.K8sValidator.allowed_network_drivers,
            default_network_driver=validator_default_network_driver
            or v.K8sValidator.default_network_driver,
        ):
            if assert_raised:
                self.assertRaises(
                    exception.InvalidParameterValue,
                    test,
                    self,
                    cluster_template,
                )
            else:
                test(self, cluster_template)
        return cluster_template

    def test_enforce_network_driver_types_one_allowed_create(self):
        self._test_enforce_network_driver_types_create(
            network_driver_type="type1",
            validator_allowed_network_drivers=["type1"],
        )

    def test_enforce_network_driver_types_two_allowed_create(self):
        self._test_enforce_network_driver_types_create(
            network_driver_type="type1",
            validator_allowed_network_drivers=["type1", "type2"],
        )

    def test_enforce_network_driver_types_not_allowed_create(self):
        self._test_enforce_network_driver_types_create(
            network_driver_type="type1",
            validator_allowed_network_drivers=["type2"],
            assert_raised=True,
        )

    def test_enforce_network_driver_types_all_allowed_create(self):
        for driver in ['flannel', 'type1', 'type2']:
            self._test_enforce_network_driver_types_create(
                network_driver_type=driver,
                validator_allowed_network_drivers=["all"],
            )

    def test_enforce_network_driver_types_invalid_coe_create(self):
        self._test_enforce_network_driver_types_create(
            network_driver_type="flannel",
            coe="invalid_coe_type",
            assert_raised=True,
        )

    def test_enforce_network_driver_types_default_create(self):
        cluster_template = self._test_enforce_network_driver_types_create(
            network_driver_type=None
        )
        self.assertEqual("flannel", cluster_template.network_driver)

    def test_enforce_network_driver_types_default_config_create(self):
        cluster_template = self._test_enforce_network_driver_types_create(
            network_driver_type=None,
            validator_allowed_network_drivers=["type1"],
            validator_default_network_driver="type1",
        )
        self.assertEqual("type1", cluster_template.network_driver)

    def test_enforce_network_driver_types_default_invalid_create(self):
        self._test_enforce_network_driver_types_create(
            network_driver_type=None,
            validator_default_network_driver="invalid_driver",
            assert_raised=True,
        )

    @mock.patch('pecan.request')
    @mock.patch('magnum.api.utils.get_resource')
    def _test_enforce_network_driver_types_update(
        self,
        mock_get_resource,
        mock_pecan_request,
        network_driver_type,
        validator_allowed_network_drivers=None,
        assert_raised=False,
    ):
        @v.enforce_network_driver_types_update()
        def test(self, cluster_template_ident, patch):
            pass

        cluster_template_ident = "test_uuid_or_name"

        patch = [{'path': '/network_driver', 'value': network_driver_type,
                  'op': 'replace'}]
        context = mock_pecan_request.context
        cluster_template = obj_utils.get_test_cluster_template(
            context, uuid=cluster_template_ident, coe='kubernetes')
        cluster_template.network_driver = network_driver_type
        mock_get_resource.return_value = cluster_template

        # NOTE(dalees): Patch the validator class variables directly, so the
        #               changes are removed after the test.
        with mock.patch.multiple(
            v.K8sValidator,
            supported_network_drivers=["flannel", "type1", "type2"],
            allowed_network_drivers=validator_allowed_network_drivers
            or v.K8sValidator.allowed_network_drivers,
        ):
            if assert_raised:
                self.assertRaises(
                    exception.InvalidParameterValue,
                    test,
                    self,
                    cluster_template_ident,
                    patch,
                )
            else:
                test(self, cluster_template_ident, patch)
                mock_get_resource.assert_called_once_with(
                    "ClusterTemplate", cluster_template_ident
                )

    def test_enforce_network_driver_types_one_allowed_update(self):
        self._test_enforce_network_driver_types_update(
            network_driver_type="type1",
            validator_allowed_network_drivers=["type1"],
        )

    def test_enforce_network_driver_types_two_allowed_update(self):
        self._test_enforce_network_driver_types_update(
            network_driver_type="type1",
            validator_allowed_network_drivers=["type1", "type2"],
        )

    def test_enforce_network_driver_types_not_allowed_update(self):
        self._test_enforce_network_driver_types_update(
            network_driver_type="type1",
            validator_allowed_network_drivers=["type2"],
            assert_raised=True,
        )

    def test_enforce_network_driver_types_all_allowed_update(self):
        for driver in ['flannel', 'type1', 'type2']:
            self._test_enforce_network_driver_types_update(
                network_driver_type=driver,
                validator_allowed_network_drivers=["all"],
            )

    def _test_enforce_volume_driver_types_create(
            self,
            volume_driver_type,
            coe='kubernetes',
            assert_raised=False):

        @v.enforce_volume_driver_types_create()
        def test(self, cluster_template):
            pass

        cluster_template = obj_utils.get_test_cluster_template(
            {}, name='test_cluster_template', coe=coe,
            volume_driver=volume_driver_type)

        if assert_raised:
            self.assertRaises(exception.InvalidParameterValue,
                              test, self, cluster_template)
        else:
            test(self, cluster_template)

    def test_enforce_volume_driver_types_valid_create(self):
        self._test_enforce_volume_driver_types_create(
            volume_driver_type='cinder')

    def test_enforce_volume_driver_types_invalid_create(self):
        self._test_enforce_volume_driver_types_create(
            volume_driver_type='type',
            assert_raised=True)

    def _test_enforce_server_type(
            self,
            server_type,
            coe='kubernetes',
            assert_raised=False):

        @v.enforce_server_type()
        def test(self, cluster_template):
            pass

        cluster_template = obj_utils.get_test_cluster_template(
            {}, name='test_cluster_template', coe=coe,
            server_type=server_type)

        if assert_raised:
            self.assertRaises(exception.InvalidParameterValue,
                              test, self, cluster_template)
        else:
            test(self, cluster_template)

    def test_enforce_server_type_valid_vm(self):
        self._test_enforce_server_type(
            server_type='vm')

    def test_enforce_server_type_valid_bm(self):
        self._test_enforce_server_type(
            server_type='bm')

    def test_enforce_server_type_invalid(self):
        self._test_enforce_server_type(
            server_type='invalid',
            assert_raised=True)

    @mock.patch('pecan.request')
    @mock.patch('magnum.api.utils.get_resource')
    def _test_enforce_volume_driver_types_update(
            self,
            mock_get_resource,
            mock_pecan_request,
            volume_driver_type,
            op,
            assert_raised=False):

        @v.enforce_volume_driver_types_update()
        def test(self, cluster_template_ident, patch):
            pass

        cluster_template_ident = 'test_uuid_or_name'
        patch = [{'path': '/volume_driver', 'value': volume_driver_type,
                  'op': op}]
        context = mock_pecan_request.context
        cluster_template = obj_utils.get_test_cluster_template(
            context, uuid=cluster_template_ident, coe='kubernetes')
        mock_get_resource.return_value = cluster_template

        # Reload the validator module so that ClusterTemplate configs are
        # re-evaluated.
        importlib.reload(v)
        validator = v.K8sValidator

        with mock.patch.multiple(
            validator, supported_volume_driver=["cinder"]
        ):
            if assert_raised:
                self.assertRaises(
                    exception.InvalidParameterValue,
                    test,
                    self,
                    cluster_template_ident,
                    patch,
                )
            else:
                test(self, cluster_template_ident, patch)
                mock_get_resource.assert_called_once_with(
                    "ClusterTemplate", cluster_template_ident
                )

    def test_enforce_volume_driver_types_supported_replace_update(self):
        self._test_enforce_volume_driver_types_update(
            volume_driver_type='cinder',
            op='replace')

    def test_enforce_volume_driver_types_not_supported_replace_update(self):
        self._test_enforce_volume_driver_types_update(
            volume_driver_type='type1',
            op='replace',
            assert_raised=True)

    def test_enforce_volume_driver_types_supported_add_update(self):
        self._test_enforce_volume_driver_types_update(
            volume_driver_type='cinder',
            op='add')

    def test_enforce_volume_driver_types_not_supported_add_update(self):
        self._test_enforce_volume_driver_types_update(
            volume_driver_type='type1',
            op='add',
            assert_raised=True)

    def test_enforce_volume_driver_types_remove_update(self):
        self._test_enforce_volume_driver_types_update(
            volume_driver_type='cinder',
            op='remove')

    def test_validate_cluster_properties(self):
        allowed_properties = v.cluster_update_allowed_properties
        for field in objects.Cluster.fields:
            if field in allowed_properties:
                v.validate_cluster_properties(set([field]))
            else:
                self.assertRaises(exception.InvalidParameterValue,
                                  v.validate_cluster_properties, set([field]))



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from unittest import mock

from oslo_utils import uuidutils

from magnum.common import context as magnum_context
from magnum.conductor import api as rpcapi
import magnum.conf
from magnum.tests.unit.api import base as api_base
from magnum.tests.unit.objects import utils as obj_utils

CONF = magnum.conf.CONF


class TestClusterResize(api_base.FunctionalTest):

    def setUp(self):
        super(TestClusterResize, self).setUp()
        self.cluster_obj = obj_utils.create_test_cluster(
            self.context, name='cluster_example_A', node_count=3)
        p = mock.patch.object(rpcapi.API, 'cluster_resize_async')
        self.mock_cluster_resize = p.start()
        self.mock_cluster_resize.side_effect = self._sim_rpc_cluster_resize
        self.addCleanup(p.stop)

    def _sim_rpc_cluster_resize(self, cluster, node_count, nodes_to_remove,
                                nodegroup, rollback=False):
        nodegroup.node_count = node_count
        nodegroup.save()
        return cluster

    def test_resize(self):
        new_node_count = 6
        response = self.post_json('/clusters/%s/actions/resize' %
                                  self.cluster_obj.uuid,
                                  {"node_count": new_node_count},
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.7",
                                           "X-Roles": "member"})
        self.assertEqual(202, response.status_code)

        response = self.get_json('/clusters/%s' % self.cluster_obj.uuid)
        self.assertEqual(new_node_count, response['node_count'])
        self.assertEqual(self.cluster_obj.uuid, response['uuid'])
        self.assertEqual(self.cluster_obj.cluster_template_id,
                         response['cluster_template_id'])

    def test_resize_with_nodegroup(self):
        new_node_count = 6
        nodegroup = self.cluster_obj.default_ng_worker
        # Verify that the API is ok with maximum allowed
        # node count set to None
        self.assertIsNone(nodegroup.max_node_count)
        cluster_resize_req = {
            "node_count": new_node_count,
            "nodegroup": nodegroup.uuid
        }
        response = self.post_json('/clusters/%s/actions/resize' %
                                  self.cluster_obj.uuid,
                                  cluster_resize_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"})
        self.assertEqual(202, response.status_code)

        response = self.get_json('/clusters/%s' % self.cluster_obj.uuid)
        self.assertEqual(new_node_count, response['node_count'])
        self.assertEqual(self.cluster_obj.uuid, response['uuid'])
        self.assertEqual(self.cluster_obj.cluster_template_id,
                         response['cluster_template_id'])

    def test_resize_with_master_nodegroup_even_unsupported(self):
        new_node_count = 4
        nodegroup = self.cluster_obj.default_ng_master
        cluster_resize_req = {
            "node_count": new_node_count,
            "nodegroup": nodegroup.uuid
        }
        response = self.post_json('/clusters/%s/actions/resize' %
                                  self.cluster_obj.uuid,
                                  cluster_resize_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"},
                                  expect_errors=True)
        self.assertEqual(400, response.status_code)

    def test_resize_with_master_nodegroup_odd_unsupported(self):
        new_node_count = 3
        nodegroup = self.cluster_obj.default_ng_master
        cluster_resize_req = {
            "node_count": new_node_count,
            "nodegroup": nodegroup.uuid
        }
        response = self.post_json('/clusters/%s/actions/resize' %
                                  self.cluster_obj.uuid,
                                  cluster_resize_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"},
                                  expect_errors=True)
        self.assertEqual(400, response.status_code)

    def test_resize_with_node_count_greater_than_max(self):
        new_node_count = 6
        nodegroup = self.cluster_obj.default_ng_worker
        nodegroup.max_node_count = 5
        nodegroup.save()
        cluster_resize_req = {
            "node_count": new_node_count,
            "nodegroup": nodegroup.uuid
        }
        response = self.post_json('/clusters/%s/actions/resize' %
                                  self.cluster_obj.uuid,
                                  cluster_resize_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"},
                                  expect_errors=True)
        self.assertEqual(400, response.status_code)

    def test_resize_with_node_count_less_than_min(self):
        new_node_count = 3
        nodegroup = self.cluster_obj.default_ng_worker
        nodegroup.min_node_count = 4
        nodegroup.save()
        cluster_resize_req = {
            "node_count": new_node_count,
            "nodegroup": nodegroup.uuid
        }
        response = self.post_json('/clusters/%s/actions/resize' %
                                  self.cluster_obj.uuid,
                                  cluster_resize_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"},
                                  expect_errors=True)
        self.assertEqual(400, response.status_code)

    def test_resize_with_zero_node_count_fail(self):
        new_node_count = 0
        nodegroup = self.cluster_obj.default_ng_worker
        nodegroup.min_node_count = 0
        nodegroup.save()
        cluster_resize_req = {
            "node_count": new_node_count,
            "nodegroup": nodegroup.uuid
        }
        response = self.post_json('/clusters/%s/actions/resize' %
                                  self.cluster_obj.uuid,
                                  cluster_resize_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"},
                                  expect_errors=True)
        self.assertEqual(400, response.status_code)

    def test_resize_with_zero_node_count(self):
        new_node_count = 0
        nodegroup = self.cluster_obj.default_ng_worker
        nodegroup.min_node_count = 0
        nodegroup.save()
        cluster_resize_req = {
            "node_count": new_node_count,
            "nodegroup": nodegroup.uuid
        }
        response = self.post_json('/clusters/%s/actions/resize' %
                                  self.cluster_obj.uuid,
                                  cluster_resize_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.10",
                                           "X-Roles": "member"})
        self.assertEqual(202, response.status_code)


class TestClusterUpgrade(api_base.FunctionalTest):
    def setUp(self):
        super(TestClusterUpgrade, self).setUp()
        self.cluster_template1 = obj_utils.create_test_cluster_template(
            self.context, uuid='94889766-e686-11e9-81b4-2a2ae2dbcce4',
            name='test_1', id=1)
        self.cluster_template2 = obj_utils.create_test_cluster_template(
            self.context, uuid='94889aa4-e686-11e9-81b4-2a2ae2dbcce4',
            name='test_2', id=2)
        self.cluster_obj = obj_utils.create_test_cluster(
            self.context, name='cluster_example_A',
            cluster_template_id=self.cluster_template1.uuid)
        self.nodegroup_obj = obj_utils.create_test_nodegroup(
            self.context, name='test_ng', cluster_id=self.cluster_obj.uuid,
            uuid='27e3153e-d5bf-4b7e-b517-fb518e17f34c',
            project_id=self.cluster_obj.project_id,
            is_default=False)
        p = mock.patch.object(rpcapi.API, 'cluster_upgrade')
        self.mock_cluster_upgrade = p.start()
        self.mock_cluster_upgrade.side_effect = self._sim_rpc_cluster_upgrade
        self.addCleanup(p.stop)

    def _sim_rpc_cluster_upgrade(self, cluster, cluster_template, batch_size,
                                 nodegroup):
        return cluster

    def test_upgrade(self):
        cluster_upgrade_req = {
            "cluster_template": "test_2"
        }
        response = self.post_json('/clusters/%s/actions/upgrade' %
                                  self.cluster_obj.uuid,
                                  cluster_upgrade_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.8",
                                           "X-Roles": "member"})
        self.assertEqual(202, response.status_code)

    def test_upgrade_cluster_as_admin(self):
        token_info = {
            'token': {
                'project': {'id': 'fake_project_1'},
                'user': {'id': 'fake_user_1'}
            }
        }
        user_context = magnum_context.RequestContext(
            auth_token_info=token_info,
            project_id='fake_project_1',
            user_id='fake_user_1',
            is_admin=False)
        cluster_uuid = uuidutils.generate_uuid()
        cluster_template_uuid = uuidutils.generate_uuid()
        obj_utils.create_test_cluster_template(
            user_context,
            public=True, uuid=cluster_template_uuid)
        obj_utils.create_test_cluster(
            user_context,
            uuid=cluster_uuid,
            cluster_template_id=cluster_template_uuid)

        cluster_upgrade_req = {"cluster_template": "test_2"}
        self.context.is_admin = True
        response = self.post_json(
            '/clusters/%s/actions/upgrade' %
            cluster_uuid,
            cluster_upgrade_req,
            headers={"Openstack-Api-Version": "container-infra 1.8",
                     "X-Roles": "member"})

        self.assertEqual(202, response.status_int)

    def test_upgrade_default_worker(self):
        cluster_upgrade_req = {
            "cluster_template": "test_2",
            "nodegroup": self.cluster_obj.default_ng_worker.uuid
        }
        response = self.post_json('/clusters/%s/actions/upgrade' %
                                  self.cluster_obj.uuid,
                                  cluster_upgrade_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"})
        self.assertEqual(202, response.status_code)

    def test_upgrade_default_master(self):
        cluster_upgrade_req = {
            "cluster_template": "test_2",
            "nodegroup": self.cluster_obj.default_ng_master.uuid
        }
        response = self.post_json('/clusters/%s/actions/upgrade' %
                                  self.cluster_obj.uuid,
                                  cluster_upgrade_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"})
        self.assertEqual(202, response.status_code)

    def test_upgrade_non_default_ng(self):
        cluster_upgrade_req = {
            "cluster_template": "test_1",
            "nodegroup": self.nodegroup_obj.uuid
        }
        response = self.post_json('/clusters/%s/actions/upgrade' %
                                  self.cluster_obj.uuid,
                                  cluster_upgrade_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"})
        self.assertEqual(202, response.status_code)

    def test_upgrade_cluster_not_found(self):
        cluster_upgrade_req = {
            "cluster_template": "test_2"
        }
        response = self.post_json('/clusters/not_there/actions/upgrade',
                                  cluster_upgrade_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.8",
                                           "X-Roles": "member"},
                                  expect_errors=True)
        self.assertEqual(404, response.status_code)

    def test_upgrade_ct_not_found(self):
        cluster_upgrade_req = {
            "cluster_template": "not_there"
        }
        response = self.post_json('/clusters/%s/actions/upgrade' %
                                  self.cluster_obj.uuid,
                                  cluster_upgrade_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.8",
                                           "X-Roles": "member"},
                                  expect_errors=True)
        self.assertEqual(404, response.status_code)

    def test_upgrade_ng_not_found(self):
        cluster_upgrade_req = {
            "cluster_template": "test_2",
            "nodegroup": "not_there"
        }
        response = self.post_json('/clusters/%s/actions/upgrade' %
                                  self.cluster_obj.uuid,
                                  cluster_upgrade_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"},
                                  expect_errors=True)
        self.assertEqual(404, response.status_code)

    def test_upgrade_non_default_ng_invalid_ct(self):
        cluster_upgrade_req = {
            "cluster_template": "test_2",
            "nodegroup": self.nodegroup_obj.uuid
        }
        response = self.post_json('/clusters/%s/actions/upgrade' %
                                  self.cluster_obj.uuid,
                                  cluster_upgrade_req,
                                  headers={"Openstack-Api-Version":
                                           "container-infra 1.9",
                                           "X-Roles": "member"},
                                  expect_errors=True)
        self.assertEqual(409, response.status_code)



# Copyright 2014, 2015 Rackspace US, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Certificate manager API
"""
import abc

from magnum.common.x509 import operations


class Cert(object, metaclass=abc.ABCMeta):
    """Base class to represent all certificates."""

    @abc.abstractmethod
    def get_certificate(self):
        """Returns the certificate."""
        pass

    @abc.abstractmethod
    def get_intermediates(self):
        """Returns the intermediate certificates."""
        pass

    @abc.abstractmethod
    def get_private_key(self):
        """Returns the private key for the certificate."""
        pass

    def get_decrypted_private_key(self):
        """Returns the decrypted private key for the certificate."""
        return operations.decrypt_key(self.get_private_key(),
                                      self.get_private_key_passphrase())

    @abc.abstractmethod
    def get_private_key_passphrase(self):
        """Returns the passphrase for the private key."""
        pass


class CertManager(object, metaclass=abc.ABCMeta):
    """Base Cert Manager Interface

    A Cert Manager is responsible for managing certificates for TLS.
    """

    @abc.abstractmethod
    def store_cert(self, certificate, private_key, intermediates=None,
                   private_key_passphrase=None, expiration=None,
                   name='Magnum TLS Cert', **kwargs):
        """Stores (i.e., registers) a cert with the cert manager.

        This method stores the specified cert and returns its UUID that
        identifies it within the cert manager.
        If storage of the certificate data fails, a CertificateStorageException
        should be raised.
        """
        pass

    @abc.abstractmethod
    def get_cert(self, cert_uuid, check_only=False, **kwargs):
        """Retrieves the specified cert.

        If check_only is True, don't perform any sort of registration.
        If the specified cert does not exist, a CertificateStorageException
        should be raised.
        """
        pass

    @abc.abstractmethod
    def delete_cert(self, cert_uuid, **kwargs):
        """Deletes the specified cert.

        If the specified cert does not exist, a CertificateStorageException
        should be raised.
        """
        pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

from magnum.i18n import _

cluster_group = cfg.OptGroup(name='cluster',
                             title='Options for Cluster configuration')

cluster_def_opts = [
    cfg.StrOpt('etcd_discovery_service_endpoint_format',
               default='https://discovery.etcd.io/new?size=%(size)d',
               help=_('Url for etcd public discovery endpoint.'),
               deprecated_group='bay'),
    cfg.StrOpt('nodes_affinity_policy',
               default='soft-anti-affinity',
               help=_('Affinity policy for server group of cluster nodes. '
                      'Possible values include "affinity", "anti-affinity", '
                      '"soft-affinity" and "soft-anti-affinity".')
               ),
    cfg.StrOpt('temp_cache_dir',
               default="/var/lib/magnum/certificate-cache",
               help='Explicitly specify the temporary directory to hold '
                    'cached TLS certs.'),
    cfg.IntOpt('pre_delete_lb_timeout',
               default=60,
               help=_('The timeout in seconds to wait for the load balancers '
                      'to be deleted.')),
]


def register_opts(conf):
    conf.register_group(cluster_group)
    conf.register_opts(cluster_def_opts, group=cluster_group)


def list_opts():
    return {
        cluster_group: cluster_def_opts
    }



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""initial migration

Revision ID: 2581ebaf0cb2
Revises: None
Create Date: 2014-01-17 12:14:07.754448

"""

# revision identifiers, used by Alembic.
revision = '2581ebaf0cb2'
down_revision = None

from alembic import op  # noqa: E402

from sqlalchemy.types import String  # noqa: E402

import sqlalchemy as sa  # noqa: E402


def upgrade():
    # commands auto generated by Alembic - please adjust!
    op.create_table(
        'bay',
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=True),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('baymodel_id', sa.String(length=255), nullable=True),
        sa.Column('node_count', sa.Integer(), nullable=True),
        sa.Column('master_address', sa.String(length=255), nullable=True),
        sa.Column('minions_address', sa.Text(), nullable=True),
        sa.Column('stack_id', sa.String(length=255), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        mysql_ENGINE='InnoDB',
        mysql_DEFAULT_CHARSET='UTF8'
    )
    op.create_table(
        'baymodel',
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=True),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('flavor_id', sa.String(length=255), nullable=True),
        sa.Column('keypair_id', sa.String(length=255), nullable=True),
        sa.Column('image_id', sa.String(length=255), nullable=True),
        sa.Column('external_network_id', sa.String(length=255), nullable=True),
        sa.Column('dns_nameserver', String(255), nullable=True),
        sa.Column('apiserver_port', sa.Integer(), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        mysql_ENGINE='InnoDB',
        mysql_DEFAULT_CHARSET='UTF8'
    )
    op.create_table(
        'container',
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=True),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('image_id', sa.String(length=255), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        mysql_ENGINE='InnoDB',
        mysql_DEFAULT_CHARSET='UTF8'
    )
    op.create_table(
        'node',
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=True),
        sa.Column('type', sa.String(length=20), nullable=True),
        sa.Column('image_id', sa.String(length=255), nullable=True),
        sa.Column('ironic_node_id', sa.String(length=36), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        mysql_ENGINE='InnoDB',
        mysql_DEFAULT_CHARSET='UTF8'
    )
    op.create_table(
        'pod',
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=True),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('desc', sa.String(length=255), nullable=True),
        sa.Column('bay_uuid', sa.String(length=36), nullable=True),
        sa.Column('images', sa.Text(), nullable=False),
        sa.Column('labels', sa.Text(), nullable=True),
        sa.Column('status', sa.String(length=255), nullable=False),
        sa.PrimaryKeyConstraint('id'),
        mysql_ENGINE='InnoDB',
        mysql_DEFAULT_CHARSET='UTF8'
    )
    op.create_table(
        'service',
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=True),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('bay_uuid', sa.String(length=36), nullable=True),
        sa.Column('labels', sa.Text, nullable=True),
        sa.Column('selector', sa.Text, nullable=True),
        sa.Column('ip', sa.String(length=36), nullable=True),
        sa.Column('port', sa.Integer(), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        mysql_ENGINE='InnoDB',
        mysql_DEFAULT_CHARSET='UTF8'
    )
    op.create_table(
        'replicationcontroller',
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('uuid', sa.String(length=36), nullable=True),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('bay_uuid', sa.String(length=36), nullable=True),
        sa.Column('images', sa.Text(), nullable=False),
        sa.Column('labels', sa.Text(), nullable=True),
        sa.Column('replicas', sa.Integer(), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        mysql_ENGINE='InnoDB',
        mysql_DEFAULT_CHARSET='UTF8'
    )
    # end Alembic commands



# Copyright 2015 Huawei Technologies Co.,LTD.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from magnum.tests import base


class TestK8sAPI(base.TestCase):
    content_dict = {
        'fake-magnum-cert-ref': {
            'certificate': 'certificate-content',
            'private_key': 'private-key-content',
            'decrypted_private_key': 'private-key-content',
        },
        'fake-ca-cert-ref': {
            'certificate': 'ca-cert-content',
            'private_key': None,
            'decrypted_private_key': None,
        }
    }
    file_dict = {
        'ca-cert-content': mock.MagicMock(),
        'certificate-content': mock.MagicMock(),
        'private-key-content': mock.MagicMock()
    }
    file_name = {
        'ca-cert-content': 'ca-cert-temp-file-name',
        'certificate-content': 'cert-temp-file-name',
        'private-key-content': 'priv-key-temp-file-name'
    }

    def _mock_cert_mgr_get_cert(self, cert_ref, **kwargs):
        cert_obj = mock.MagicMock()
        cert_obj.get_certificate.return_value = (
            TestK8sAPI.content_dict[cert_ref]['certificate'])
        cert_obj.get_private_key.return_value = (
            TestK8sAPI.content_dict[cert_ref]['private_key'])
        cert_obj.get_decrypted_private_key.return_value = (
            TestK8sAPI.content_dict[cert_ref]['decrypted_private_key'])

        return cert_obj



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_versionedobjects import fields

from magnum.objects import base


@base.MagnumObjectRegistry.register
class Certificate(base.MagnumPersistentObject, base.MagnumObject):
    # Version 1.0: Initial version
    # Version 1.1: Rename bay_uuid to cluster_uuid
    # Version 1.2: Add ca_cert_type to indicate what's the CA cert type the
    #              CSR being signed

    VERSION = '1.2'

    fields = {
        'project_id': fields.StringField(nullable=True),
        'user_id': fields.StringField(nullable=True),
        'cluster_uuid': fields.StringField(nullable=True),
        'csr': fields.StringField(nullable=True),
        'pem': fields.StringField(nullable=True),
        'ca_cert_type': fields.StringField(nullable=True),
    }

    @classmethod
    def from_object_cluster(cls, cluster):
        return cls(project_id=cluster.project_id,
                   user_id=cluster.user_id,
                   cluster_uuid=cluster.uuid)

    @classmethod
    def from_db_cluster(cls, cluster):
        return cls(project_id=cluster['project_id'],
                   user_id=cluster['user_id'],
                   cluster_uuid=cluster['uuid'])



# Copyright 2015 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime
import uuid

from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography import x509
from oslo_log import log as logging

from magnum.common import exception
from magnum.common.x509 import validator
import magnum.conf

LOG = logging.getLogger(__name__)

CONF = magnum.conf.CONF


def generate_ca_certificate(subject_name, encryption_password=None):
    """Generate CA Certificate

    :param subject_name: subject name of CA
    :param encryption_password: encryption passsword for private key
    :returns: generated private key and certificate pair
    """
    return _generate_self_signed_certificate(
        subject_name,
        _build_ca_extentions(),
        encryption_password=encryption_password
    )


def generate_client_certificate(issuer_name, subject_name,
                                organization_name, ca_key,
                                encryption_password=None,
                                ca_key_password=None):
    """Generate Client Certificate

    :param issuer_name: issuer name
    :param subject_name: subject name of client
    :param organization_name: Organization name of client
    :param ca_key: private key of CA
    :param encryption_password: encryption passsword for private key
    :param ca_key_password: private key password for given ca key
    :returns: generated private key and certificate pair
    """
    return _generate_certificate(issuer_name, subject_name,
                                 _build_client_extentions(),
                                 organization_name, ca_key=ca_key,
                                 encryption_password=encryption_password,
                                 ca_key_password=ca_key_password)


def _build_client_extentions():
    # Digital Signature and Key Encipherment are enabled
    key_usage = x509.KeyUsage(True, False, True, False, False, False, False,
                              False, False)
    key_usage = x509.Extension(key_usage.oid, True, key_usage)
    extended_key_usage = x509.ExtendedKeyUsage([x509.OID_CLIENT_AUTH])
    extended_key_usage = x509.Extension(extended_key_usage.oid, False,
                                        extended_key_usage)
    basic_constraints = x509.BasicConstraints(ca=False, path_length=None)
    basic_constraints = x509.Extension(basic_constraints.oid, True,
                                       basic_constraints)

    return [key_usage, extended_key_usage, basic_constraints]


def _build_ca_extentions():
    # Certificate Sign is enabled
    key_usage = x509.KeyUsage(False, False, False, False, False, True, False,
                              False, False)
    key_usage = x509.Extension(key_usage.oid, True, key_usage)

    basic_constraints = x509.BasicConstraints(ca=True, path_length=0)
    basic_constraints = x509.Extension(basic_constraints.oid, True,
                                       basic_constraints)

    return [basic_constraints, key_usage]


def _generate_self_signed_certificate(subject_name, extensions,
                                      encryption_password=None):
    return _generate_certificate(subject_name, subject_name, extensions,
                                 encryption_password=encryption_password)


def _generate_certificate(issuer_name, subject_name, extensions,
                          organization_name=None, ca_key=None,
                          encryption_password=None, ca_key_password=None):

    if not isinstance(subject_name, str):
        subject_name = subject_name.decode('utf-8')
    if organization_name and not isinstance(organization_name, str):
        organization_name = organization_name.decode('utf-8')

    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=CONF.x509.rsa_key_size
    )

    # subject name is set as common name
    csr = x509.CertificateSigningRequestBuilder()
    name_attributes = [x509.NameAttribute(x509.OID_COMMON_NAME, subject_name)]
    if organization_name:
        name_attributes.append(x509.NameAttribute(x509.OID_ORGANIZATION_NAME,
                                                  organization_name))
    csr = csr.subject_name(x509.Name(name_attributes))

    for extention in extensions:
        csr = csr.add_extension(extention.value, critical=extention.critical)

    # if ca_key is not provided, it means self signed
    if not ca_key:
        ca_key = private_key
        ca_key_password = encryption_password

    csr = csr.sign(private_key, hashes.SHA256())

    if isinstance(encryption_password, str):
        encryption_password = encryption_password.encode('latin-1')

    if encryption_password:
        encryption_algorithm = serialization.BestAvailableEncryption(
            encryption_password)
    else:
        encryption_algorithm = serialization.NoEncryption()

    private_key = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=encryption_algorithm
    )

    keypairs = {
        'private_key': private_key,
        'certificate': sign(
            csr,
            issuer_name,
            ca_key,
            ca_key_password=ca_key_password,
            skip_validation=True),
    }
    return keypairs


def _load_pem_private_key(ca_key, ca_key_password=None):
    if not isinstance(ca_key, rsa.RSAPrivateKey):
        if isinstance(ca_key, str):
            ca_key = ca_key.encode('latin-1')
        if isinstance(ca_key_password, str):
            ca_key_password = ca_key_password.encode('latin-1')

        ca_key = serialization.load_pem_private_key(
            ca_key,
            password=ca_key_password
        )

    return ca_key


def sign(csr, issuer_name, ca_key, ca_key_password=None,
         skip_validation=False):
    """Sign a given csr

    :param csr: certificate signing request object or pem encoded csr
    :param issuer_name: issuer name
    :param ca_key: private key of CA
    :param ca_key_password: private key password for given ca key
    :param skip_validation: skip csr validation if true
    :returns: generated certificate
    """

    ca_key = _load_pem_private_key(ca_key, ca_key_password)

    if not isinstance(issuer_name, str):
        issuer_name = issuer_name.decode('utf-8')

    if isinstance(csr, str):
        csr = csr.encode('latin-1')
    if not isinstance(csr, x509.CertificateSigningRequest):
        try:
            csr = x509.load_pem_x509_csr(csr)
        except ValueError:
            LOG.exception("Received invalid csr %s.", csr)
            raise exception.InvalidCsr(csr=csr)

    term_of_validity = CONF.x509.term_of_validity
    one_day = datetime.timedelta(1, 0, 0)
    expire_after = datetime.timedelta(term_of_validity, 0, 0)

    builder = x509.CertificateBuilder()
    builder = builder.subject_name(csr.subject)
    # issuer_name is set as common name
    builder = builder.issuer_name(x509.Name([
        x509.NameAttribute(x509.OID_COMMON_NAME, issuer_name),
    ]))
    builder = builder.not_valid_before(datetime.datetime.today() - one_day)
    builder = builder.not_valid_after(datetime.datetime.today() + expire_after)
    builder = builder.serial_number(int(uuid.uuid4()))
    builder = builder.public_key(csr.public_key())

    if skip_validation:
        extensions = csr.extensions
    else:
        extensions = validator.filter_extensions(csr.extensions)

    for extention in extensions:
        builder = builder.add_extension(extention.value,
                                        critical=extention.critical)

    certificate = builder.sign(
        private_key=ca_key, algorithm=hashes.SHA256(),
    ).public_bytes(serialization.Encoding.PEM).strip()

    return certificate


def generate_csr_and_key(common_name):
    """Return a dict with a new csr, public key and private key."""
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=2048
    )

    public_key = private_key.public_key()

    csr = x509.CertificateSigningRequestBuilder().subject_name(x509.Name([
        x509.NameAttribute(x509.oid.NameOID.COMMON_NAME, common_name),
    ])).sign(private_key, hashes.SHA256())

    result = {
        'csr': csr.public_bytes(
            encoding=serialization.Encoding.PEM).decode("utf-8"),
        'private_key': private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()).decode("utf-8"),
        'public_key': public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo).decode(
                "utf-8"),
    }

    return result


def decrypt_key(encrypted_key, password):
    private_key = _load_pem_private_key(encrypted_key, password)

    decrypted_pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption()
    )
    return decrypted_pem



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

from magnum.i18n import _

quotas_group = cfg.OptGroup(name='quotas',
                            title='Options for quota configuration')

quotas_def_opts = [
    cfg.IntOpt('max_clusters_per_project',
               default=20,
               help=_('Max number of clusters allowed per project. Admin can '
                      'override this default quota for a project by setting '
                      'explicit limit in quotas DB table (using /quotas REST '
                      'API endpoint).')),
]


def register_opts(conf):
    conf.register_group(quotas_group)
    conf.register_opts(quotas_def_opts, group=quotas_group)


def list_opts():
    return {
        quotas_group: quotas_def_opts
    }



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils
from webtest.app import AppError

from magnum.tests.unit.common.policies import base
from magnum.tests.unit.objects import utils as obj_utils


class TestFederationPolicy(base.PolicyFunctionalTest):
    def setUp(self):
        super(TestFederationPolicy, self).setUp()
        self.create_frederation()

    def create_frederation(self):
        self.fake_uuid = uuidutils.generate_uuid()
        self.federation = obj_utils.create_test_federation(
            self.context, uuid=self.fake_uuid)

    def test_get_no_permission(self):
        exc = self.assertRaises(
            AppError, self.get_json, '/federations',
            headers=self.member_headers)
        self.assertIn("403 Forbidden", str(exc))

    def test_get_reader(self):
        response = self.get_json('/federations')
        self.assertEqual(self.fake_uuid, response['federations'][0]['uuid'])

    def test_create_no_permission(self):
        exc = self.assertRaises(
            AppError, self.post_json, '/federations', {},
            headers=self.reader_headers)
        self.assertIn("403 Forbidden", str(exc))

    def test_update_no_permission(self):
        new_member = obj_utils.create_test_cluster(self.context)
        exc = self.assertRaises(
            AppError, self.patch_json, '/federations/%s' % self.fake_uuid,
            [{'path': '/member_ids', 'value': new_member.uuid, 'op': 'add'}],
            headers=self.reader_headers)
        self.assertIn("403 Forbidden", str(exc))

    def test_delete_no_permission(self):
        exc = self.assertRaises(
            AppError, self.delete,
            '/federations/%s' % self.fake_uuid,
            headers=self.reader_headers
        )
        self.assertIn("403 Forbidden", str(exc))

    def test_detail_list_no_permission(self):
        exc = self.assertRaises(
            AppError, self.get_json,
            '/federations/detail',
            headers=self.member_headers)
        self.assertIn("403 Forbidden", str(exc))



# Copyright 2015 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from cryptography import x509

from magnum.common import exception
from magnum.common.x509 import extensions
import magnum.conf

try:
    # for cryptography >= 35.0.0
    from cryptography.hazmat._oid import _OID_NAMES as OID_NAMES
except ImportError:
    from cryptography.x509.oid import _OID_NAMES as OID_NAMES

_CA_KEY_USAGES = [
    extensions.KeyUsages.KEY_CERT_SIGN.value[0],
    extensions.KeyUsages.CRL_SIGN.value[0]
]

CONF = magnum.conf.CONF


def filter_extensions(extensions):
    filtered_extensions = []
    allowed_key_usage = set(CONF.x509.allowed_key_usage)
    if not CONF.x509.allow_ca:
        allowed_key_usage = _remove_ca_key_usage(allowed_key_usage)

    for ext in filter_allowed_extensions(extensions,
                                         CONF.x509.allowed_extensions):
        if ext.oid == x509.OID_KEY_USAGE:
            ext = _merge_key_usage(ext, allowed_key_usage)
        elif ext.oid == x509.OID_BASIC_CONSTRAINTS:
            if not CONF.x509.allow_ca:
                ext = _disallow_ca_in_basic_constraints(ext)

        filtered_extensions.append(ext)

    return filtered_extensions


def filter_allowed_extensions(extensions, allowed_extensions=None):
    """Ensure only accepted extensions are used."""
    allowed_extensions = allowed_extensions or []

    for ext in extensions:
        ext_name = OID_NAMES.get(ext.oid, None)
        if ext_name in allowed_extensions:
            yield ext
        else:
            if ext.critical:
                raise exception.CertificateValidationError(extension=ext)


def _merge_key_usage(key_usage, allowed_key_usage):
    critical = key_usage.critical
    key_usage_value = key_usage.value

    usages = []
    for usage in extensions.KeyUsages:
        k, v = usage.value
        try:
            value = getattr(key_usage_value, v)
        except ValueError:
            # ValueError is raised when encipher_only/decipher_only is
            # retrieved but key_agreement is False
            value = False
        if value:
            if k not in allowed_key_usage:
                if critical:
                    raise exception.CertificateValidationError(
                        extension=key_usage)
                else:
                    value = False
        usages.append(value)

    rtn = x509.KeyUsage(*usages)
    return x509.Extension(rtn.oid, critical, rtn)


def _remove_ca_key_usage(allowed_key_usage):
    for usage in _CA_KEY_USAGES:
        try:
            allowed_key_usage.remove(usage)
        except KeyError:
            pass
    return allowed_key_usage


def _disallow_ca_in_basic_constraints(basic_constraints):
    if basic_constraints.value.ca:
        if basic_constraints.critical:
            raise exception.CertificateValidationError(
                extension=basic_constraints)

        bc = x509.BasicConstraints(False, None)
        return x509.Extension(bc.oid, False, bc)

    return basic_constraints



# Copyright 2018 Catalyst Cloud Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
import re
import time

import heatclient.exc as heat_exc
from osc_lib import exceptions as osc_exc
from oslo_config import cfg
from oslo_log import log as logging

from magnum.common import clients
from magnum.common import context as magnum_context
from magnum.common import exception
from magnum.common import neutron

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


def wait_for_lb_deleted(octavia_client, deleted_lbs):
    """Wait for the loadbalancers to be deleted.

    Load balancer deletion API in Octavia is asynchronous so that the called
    needs to wait if it wants to guarantee the load balancer to be deleted.
    The timeout is necessary to avoid waiting infinitely.
    """
    timeout = CONF.cluster.pre_delete_lb_timeout
    start_time = time.time()

    while True:
        lbs = octavia_client.load_balancer_list().get("loadbalancers", [])
        lbIDs = set(
            [lb["id"]
             for lb in lbs if lb["provisioning_status"] != "DELETED"]
        )
        if not (deleted_lbs & lbIDs):
            break

        if (time.time() - timeout) > start_time:
            raise Exception("Timeout waiting for the load balancers "
                            "%s to be deleted." % deleted_lbs)

        time.sleep(1)


def _delete_loadbalancers(context, lbs, cluster, octavia_client,
                          remove_fip=False, cascade=True):
    candidates = set()

    for lb in lbs:
        status = lb["provisioning_status"]
        if status not in ["PENDING_DELETE", "DELETED"]:
            LOG.info("Deleting load balancer %s for cluster %s",
                     lb["id"], cluster.uuid)
            octavia_client.load_balancer_delete(lb["id"], cascade=cascade)
            candidates.add(lb["id"])

            if remove_fip:
                neutron.delete_floatingip(context, lb["vip_port_id"], cluster)

    return candidates


def delete_loadbalancers(context, cluster):  # noqa: C901
    """Delete loadbalancers for the cluster.

    The following load balancers are deleted:
    - The load balancers created for Kubernetes services and ingresses in
      the Kubernetes cluster.
    - The load balancers created for Kubernetes API and etcd for HA cluster.
    """
    pattern = (r'Kubernetes .+ from cluster %s' % cluster.uuid)
    lb_resource_type = "Magnum::Optional::Neutron::LBaaS::LoadBalancer"

    adm_ctx = magnum_context.get_admin_context()
    adm_clients = clients.OpenStackClients(adm_ctx)
    user_clients = clients.OpenStackClients(context)
    candidates = set()

    try:
        octavia_client_adm = adm_clients.octavia()
        heat_client = user_clients.heat()
        octavia_client = user_clients.octavia()

        # Get load balancers created for service/ingress
        lbs = octavia_client.load_balancer_list().get("loadbalancers", [])
        lbs = [lb for lb in lbs if re.match(pattern, lb["description"])]
        deleted = _delete_loadbalancers(context, lbs, cluster,
                                        octavia_client_adm, remove_fip=True)
        candidates.update(deleted)

        # NOTE (brtknr): If stack has been deleted, cluster fails to delete
        # because stack_id resolves to None. Return if that is the case.
        if not cluster.stack_id:
            return

        # Get load balancers created for Kubernetes api/etcd
        lbs = []
        try:
            lb_resources = heat_client.resources.list(
                cluster.stack_id, nested_depth=2,
                filters={"type": lb_resource_type})
        except heat_exc.HTTPNotFound:
            # NOTE(mnaser): It's possible that the stack has been deleted
            #               but Magnum still has a `stack_id` pointing.
            return
        for lb_res in lb_resources:
            lb_id = lb_res.physical_resource_id
            if not lb_id:
                continue
            try:
                lb = octavia_client.load_balancer_show(lb_id)
                lbs.append(lb)
            except osc_exc.NotFound:
                continue
        deleted = _delete_loadbalancers(context, lbs, cluster,
                                        octavia_client_adm, remove_fip=False)
        candidates.update(deleted)

        if not candidates:
            return

        wait_for_lb_deleted(octavia_client, candidates)
    except Exception as e:
        raise exception.PreDeletionFailed(cluster_uuid=cluster.uuid,
                                          msg=str(e))



#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""API for interfacing with Magnum Backend."""

from magnum.common import profiler
from magnum.common import rpc_service
import magnum.conf

CONF = magnum.conf.CONF

# The Backend API class serves as a AMQP client for communicating
# on a topic exchange specific to the conductors.  This allows the ReST
# API to trigger operations on the conductors


@profiler.trace_cls("rpc")
class API(rpc_service.API):
    def __init__(self, context=None, topic=CONF.conductor.topic):
        super(API, self).__init__(context=context, topic=topic)

    # Cluster Operations

    def cluster_create(self, cluster, master_count, node_count,
                       create_timeout):
        return self._call('cluster_create', cluster=cluster,
                          master_count=master_count, node_count=node_count,
                          create_timeout=create_timeout)

    def cluster_create_async(self, cluster, master_count, node_count,
                             create_timeout):
        self._cast('cluster_create', cluster=cluster,
                   master_count=master_count, node_count=node_count,
                   create_timeout=create_timeout)

    def cluster_delete(self, uuid):
        return self._call('cluster_delete', uuid=uuid)

    def cluster_delete_async(self, uuid):
        self._cast('cluster_delete', uuid=uuid)

    def cluster_update(self, cluster, node_count,
                       health_status, health_status_reason):
        return self._call(
            'cluster_update', cluster=cluster, node_count=node_count,
            health_status=health_status,
            health_status_reason=health_status_reason)

    def cluster_update_async(self, cluster, node_count,
                             health_status, health_status_reason,
                             rollback=False):
        self._cast('cluster_update', cluster=cluster,
                   node_count=node_count,
                   health_status=health_status,
                   health_status_reason=health_status_reason,
                   rollback=rollback)

    def cluster_resize(self, cluster, node_count, nodes_to_remove,
                       nodegroup, rollback=False):

        return self._call('cluster_resize',
                          cluster=cluster,
                          node_count=node_count,
                          nodes_to_remove=nodes_to_remove,
                          nodegroup=nodegroup)

    def cluster_resize_async(self, cluster, node_count, nodes_to_remove,
                             nodegroup, rollback=False):
        return self._cast('cluster_resize',
                          cluster=cluster,
                          node_count=node_count,
                          nodes_to_remove=nodes_to_remove,
                          nodegroup=nodegroup)

    def cluster_upgrade(self, cluster, cluster_template, max_batch_size,
                        nodegroup):
        return self._call('cluster_upgrade',
                          cluster=cluster,
                          cluster_template=cluster_template,
                          max_batch_size=max_batch_size,
                          nodegroup=nodegroup)

    def cluster_upgrade_async(self, cluster, cluster_template, max_batch_size,
                              nodegroup):
        return self._call('cluster_upgrade',
                          cluster=cluster,
                          cluster_template=cluster_template,
                          max_batch_size=max_batch_size,
                          nodegroup=nodegroup)

    # Federation Operations

    def federation_create(self, federation, create_timeout):
        return self._call('federation_create', federation=federation,
                          create_timeout=create_timeout)

    def federation_create_async(self, federation, create_timeout):
        self._cast('federation_create', federation=federation,
                   create_timeout=create_timeout)

    def federation_delete(self, uuid):
        return self._call('federation_delete', uuid=uuid)

    def federation_delete_async(self, uuid):
        self._cast('federation_delete', uuid=uuid)

    def federation_update(self, federation):
        return self._call('federation_update', federation=federation)

    def federation_update_async(self, federation, rollback=False):
        self._cast('federation_update', federation=federation,
                   rollback=rollback)

    # CA operations

    def sign_certificate(self, cluster, certificate):
        return self._call('sign_certificate', cluster=cluster,
                          certificate=certificate)

    def get_ca_certificate(self, cluster, ca_cert_type=None):
        return self._call('get_ca_certificate', cluster=cluster,
                          ca_cert_type=ca_cert_type)

    def rotate_ca_certificate(self, cluster):
        return self._call('rotate_ca_certificate', cluster=cluster)

    # Versioned Objects indirection API

    def object_class_action(self, context, objname, objmethod, objver,
                            args, kwargs):
        "Indirection API callback"
        return self._client.call(context, 'object_class_action',
                                 objname=objname, objmethod=objmethod,
                                 objver=objver, args=args, kwargs=kwargs)

    def object_action(self, context, objinst, objmethod, args, kwargs):
        "Indirection API callback"
        return self._client.call(context, 'object_action', objinst=objinst,
                                 objmethod=objmethod, args=args, kwargs=kwargs)

    def object_backport(self, context, objinst, target_version):
        "Indirection API callback"
        return self._client.call(context, 'object_backport', objinst=objinst,
                                 target_version=target_version)

    # NodeGroup Operations

    def nodegroup_create(self, cluster, nodegroup):
        return self._call('nodegroup_create', cluster=cluster,
                          nodegroup=nodegroup)

    def nodegroup_create_async(self, cluster, nodegroup):
        self._cast('nodegroup_create', cluster=cluster, nodegroup=nodegroup)

    def nodegroup_delete(self, cluster, nodegroup):
        return self._call('nodegroup_delete', cluster=cluster,
                          nodegroup=nodegroup)

    def nodegroup_delete_async(self, cluster, nodegroup):
        self._cast('nodegroup_delete', cluster=cluster, nodegroup=nodegroup)

    def nodegroup_update(self, cluster, nodegroup):
        return self._call('nodegroup_update', cluster=cluster,
                          nodegroup=nodegroup)

    def nodegroup_update_async(self, cluster, nodegroup):
        self._cast('nodegroup_update', cluster=cluster, nodegroup=nodegroup)


@profiler.trace_cls("rpc")
class ListenerAPI(rpc_service.API):
    def __init__(self, context=None, topic=None, server=None, timeout=None):
        super(ListenerAPI, self).__init__(context=context, topic=topic,
                                          server=server, timeout=timeout)

    def ping_conductor(self):
        return self._call('ping_conductor')



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

cluster_heat_group = cfg.OptGroup(name='cluster_heat',
                                  title='Heat options for Cluster '
                                        'configuration')

cluster_heat_opts = [
    cfg.IntOpt('max_attempts',
               default=2000,
               help=('Number of attempts to query the Heat stack for '
                     'finding out the status of the created stack and '
                     'getting template outputs.  This value is ignored '
                     'during cluster creation if timeout is set as the poll '
                     'will continue until cluster creation either ends '
                     'or times out.'),
               ),
    cfg.IntOpt('wait_interval',
               default=1,
               help=('Sleep time interval between two attempts of querying '
                     'the Heat stack.  This interval is in seconds.'),
               ),
    cfg.IntOpt('create_timeout',
               default=60,
               help=('The length of time to let cluster creation continue. '
                     'This interval is in minutes. The default is 60 minutes.'
                     ),
               )
]


def register_opts(conf):
    conf.register_group(cluster_heat_group)
    conf.register_opts(cluster_heat_opts, group=cluster_heat_group)


def list_opts():
    return {
        cluster_heat_group: cluster_heat_opts
    }



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Tests for manipulating Federations via the DB API"""
from oslo_utils import uuidutils

from magnum.common import context
from magnum.common import exception
from magnum.tests.unit.db import base
from magnum.tests.unit.db import utils


class DbFederationTestCase(base.DbTestCase):
    def test_create_federation(self):
        utils.create_test_federation()

    def test_create_federation_already_exists(self):
        utils.create_test_federation()
        self.assertRaises(exception.FederationAlreadyExists,
                          utils.create_test_federation)

    def test_get_federation_by_id(self):
        federation = utils.create_test_federation()
        res = self.dbapi.get_federation_by_id(self.context, federation.id)
        self.assertEqual(federation.id, res.id)
        self.assertEqual(federation.uuid, res.uuid)

    def test_get_federation_by_name(self):
        federation = utils.create_test_federation()
        res = self.dbapi.get_federation_by_name(self.context, federation.name)
        self.assertEqual(federation.name, res.name)
        self.assertEqual(federation.uuid, res.uuid)

    def test_get_federation_by_uuid(self):
        federation = utils.create_test_federation()
        res = self.dbapi.get_federation_by_uuid(self.context, federation.uuid)
        self.assertEqual(federation.id, res.id)
        self.assertEqual(federation.uuid, res.uuid)

    def test_get_federation_that_does_not_exist(self):
        self.assertRaises(exception.FederationNotFound,
                          self.dbapi.get_federation_by_id,
                          self.context, 999)
        self.assertRaises(exception.FederationNotFound,
                          self.dbapi.get_federation_by_uuid,
                          self.context,
                          '12345678-9999-0000-aaaa-123456789012')
        self.assertRaises(exception.FederationNotFound,
                          self.dbapi.get_federation_by_name,
                          self.context, 'not_found')

    def test_get_federation_by_name_multiple_federation(self):
        utils.create_test_federation(id=1, name='federation-1',
                                     uuid=uuidutils.generate_uuid())
        utils.create_test_federation(id=2, name='federation-1',
                                     uuid=uuidutils.generate_uuid())
        self.assertRaises(exception.Conflict,
                          self.dbapi.get_federation_by_name,
                          self.context, 'federation-1')

    def test_get_federation_list(self):
        uuids = []
        for _ in range(5):
            federation = utils.create_test_federation(
                uuid=uuidutils.generate_uuid())
            uuids.append(str(federation.uuid))

        res = self.dbapi.get_federation_list(self.context, sort_key='uuid')
        res_uuids = [r.uuid for r in res]
        self.assertEqual(sorted(uuids), res_uuids)

    def test_get_federation_list_sorted(self):
        uuids = []
        for _ in range(5):
            federation = utils.create_test_federation(
                uuid=uuidutils.generate_uuid())
            uuids.append(str(federation.uuid))

        res = self.dbapi.get_federation_list(self.context, sort_key='uuid')
        res_uuids = [r.uuid for r in res]
        self.assertEqual(sorted(uuids), res_uuids)

        self.assertRaises(exception.InvalidParameterValue,
                          self.dbapi.get_federation_list,
                          self.context,
                          sort_key='foo')

    def test_get_federation_list_with_filters(self):
        fed1 = utils.create_test_federation(
            id=1,
            uuid=uuidutils.generate_uuid(),
            name='fed1',
            project_id='proj1',
            hostcluster_id='master1',
            member_ids=['member1', 'member2'],
            properties={'dns-zone': 'fed1.com.'})

        fed2 = utils.create_test_federation(
            id=2,
            uuid=uuidutils.generate_uuid(),
            name='fed',
            project_id='proj2',
            hostcluster_id='master2',
            member_ids=['member3', 'member4'],
            properties={"dns-zone": "fed2.com."})

        # NOTE(clenimar): we are specifying a project_id to the test
        # resources above, which means that our current context
        # (self.context) will not be able to see these resources.
        # Create an admin context in order to test the queries:
        ctx = context.make_admin_context(all_tenants=True)

        # Filter by name:
        res = self.dbapi.get_federation_list(ctx, filters={'name': 'fed1'})
        self.assertEqual([fed1.id], [r.id for r in res])

        res = self.dbapi.get_federation_list(ctx, filters={'name': 'foo'})
        self.assertEqual([], [r.id for r in res])

        # Filter by project_id
        res = self.dbapi.get_federation_list(ctx,
                                             filters={'project_id': 'proj1'})
        self.assertEqual([fed1.id], [r.id for r in res])

        res = self.dbapi.get_federation_list(ctx,
                                             filters={'project_id': 'foo'})
        self.assertEqual([], [r.id for r in res])

        # Filter by hostcluster_id
        res = self.dbapi.get_federation_list(ctx, filters={
            'hostcluster_id': 'master1'})
        self.assertEqual([fed1.id], [r.id for r in res])

        res = self.dbapi.get_federation_list(ctx, filters={
            'hostcluster_id': 'master2'})
        self.assertEqual([fed2.id], [r.id for r in res])

        res = self.dbapi.get_federation_list(ctx,
                                             filters={'hostcluster_id': 'foo'})
        self.assertEqual([], [r.id for r in res])

        # Filter by member_ids (please note that it is currently implemented
        # as an exact match. So it will only return federations whose member
        # clusters are exactly those passed as a filter)
        res = self.dbapi.get_federation_list(
            ctx, filters={'member_ids': ['member1', 'member2']})
        self.assertEqual([fed1.id], [r.id for r in res])

        res = self.dbapi.get_federation_list(
            ctx, filters={'member_ids': ['foo']})
        self.assertEqual([], [r.id for r in res])

        # Filter by properties
        res = self.dbapi.get_federation_list(
            ctx, filters={
                'properties': {'dns-zone': 'fed2.com.'}
            })
        self.assertEqual([fed2.id], [r.id for r in res])

        res = self.dbapi.get_federation_list(
            ctx, filters={
                'properties': {'dns-zone': 'foo.bar.'}
            })
        self.assertEqual([], [r.id for r in res])

    def test_get_federation_list_by_admin_all_tenants(self):
        uuids = []
        for _ in range(5):
            federation = utils.create_test_federation(
                uuid=uuidutils.generate_uuid(),
                project_id=uuidutils.generate_uuid())
            uuids.append(str(federation['uuid']))

        ctx = context.make_admin_context(all_tenants=True)
        res = self.dbapi.get_federation_list(ctx)
        res_uuids = [r.uuid for r in res]
        self.assertEqual(len(res), 5)
        self.assertEqual(sorted(uuids), sorted(res_uuids))

    def test_destroy_federation(self):
        federation = utils.create_test_federation()
        self.assertIsNotNone(
            self.dbapi.get_federation_by_id(self.context, federation.id))
        self.dbapi.destroy_federation(federation.id)
        self.assertRaises(exception.FederationNotFound,
                          self.dbapi.get_federation_by_id,
                          self.context, federation.id)

    def test_destroy_federation_by_uuid(self):
        federation = utils.create_test_federation(
            uuid=uuidutils.generate_uuid())
        self.assertIsNotNone(
            self.dbapi.get_federation_by_uuid(self.context, federation.uuid))
        self.dbapi.destroy_federation(federation.uuid)
        self.assertRaises(exception.FederationNotFound,
                          self.dbapi.get_federation_by_uuid,
                          self.context, federation.uuid)

    def test_destroy_federation_by_id_that_does_not_exist(self):
        self.assertRaises(exception.FederationNotFound,
                          self.dbapi.destroy_federation,
                          '12345678-9999-0000-aaaa-123456789012')

    def test_destroy_federation_by_uudid_that_does_not_exist(self):
        self.assertRaises(exception.FederationNotFound,
                          self.dbapi.destroy_federation, '15')

    def test_update_federation_members(self):
        federation = utils.create_test_federation()
        old_members = federation.member_ids
        new_members = old_members + ['new-member-id']
        self.assertNotEqual(old_members, new_members)
        res = self.dbapi.update_federation(federation.id,
                                           {'member_ids': new_members})
        self.assertEqual(new_members, res.member_ids)

    def test_update_federation_properties(self):
        federation = utils.create_test_federation()
        old_properties = federation.properties
        new_properties = {
            'dns-zone': 'new.domain.com.'
        }
        self.assertNotEqual(old_properties, new_properties)
        res = self.dbapi.update_federation(federation.id,
                                           {'properties': new_properties})
        self.assertEqual(new_properties, res.properties)

    def test_update_federation_not_found(self):
        federation_uuid = uuidutils.generate_uuid()
        self.assertRaises(exception.FederationNotFound,
                          self.dbapi.update_federation, federation_uuid,
                          {'member_ids': ['foo']})



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import warnings

from tempest import config

from oslo_config import cfg

CONF = config.CONF


class Config(object):

    """Parses configuration to attributes required for auth and test data"""

    @classmethod
    def set_admin_creds(cls, config):
        cls.admin_user = CONF.auth.admin_username
        cls.admin_passwd = CONF.auth.admin_password
        # NOTE(toabctl): also allow the old style tempest definition
        try:
            cls.admin_tenant = CONF.auth.admin_project_name
        except cfg.NoSuchOptError:
            cls.admin_tenant = CONF.auth.admin_tenant_name
            warnings.warn("the config option 'admin_tenant_name' from the "
                          "'auth' section is deprecated. Please switch "
                          "to 'admin_project_name'.")

    @classmethod
    def set_user_creds(cls, config):
        # normal user creds
        # Fixme(eliqiao): this is quick workaround to passing tempest
        # legacy credentials provider is removed by tempest
        # I8c24cd17f643083dde71ab2bd2a38417c54aeccb.
        # TODO(eliqiao): find a way to using an accounts.yaml file
        # check Ia5132c5cb32355d6f26b8acdd92a0e55a2c19f41
        cls.user = CONF.auth.admin_username
        cls.passwd = CONF.auth.admin_password
        # NOTE(toabctl): also allow the old style tempest definition
        try:
            cls.tenant = CONF.auth.admin_project_name
        except cfg.NoSuchOptError:
            cls.tenant = CONF.auth.admin_tenant_name
            warnings.warn("the config option 'admin_tenant_name' from the "
                          "'auth' section is deprecated. Please switch "
                          "to 'admin_project_name'.")

    @classmethod
    def set_auth_version(cls, config):
        # auth version for client authentication
        cls.auth_version = CONF.identity.auth_version

    @classmethod
    def set_auth_url(cls, config):
        # auth_url for client authentication
        if cls.auth_version == 'v3':
            cls.auth_v3_url = CONF.identity.uri_v3
        else:
            if 'uri' not in CONF.identity:
                raise Exception('config missing auth_url key')
            cls.auth_url = CONF.identity.uri

    @classmethod
    def set_admin_role(cls, config):
        # admin_role for client authentication
        if cls.auth_version == 'v3':
            cls.admin_role = CONF.identity.admin_role
        else:
            cls.admin_role = 'admin'

    @classmethod
    def set_region(cls, config):
        if 'region' in CONF.identity:
            cls.region = CONF.identity.region
        else:
            cls.region = 'RegionOne'

    @classmethod
    def set_image_id(cls, config):
        if 'image_id' not in CONF.magnum:
            raise Exception('config missing image_id key')
        cls.image_id = CONF.magnum.image_id

    @classmethod
    def set_nic_id(cls, config):
        if 'nic_id' not in CONF.magnum:
            raise Exception('config missing nic_id key')
        cls.nic_id = CONF.magnum.nic_id

    @classmethod
    def set_keypair_id(cls, config):
        if 'keypair_id' not in CONF.magnum:
            raise Exception('config missing keypair_id key')
        cls.keypair_id = CONF.magnum.keypair_id

    @classmethod
    def set_flavor_id(cls, config):
        if 'flavor_id' not in CONF.magnum:
            raise Exception('config missing flavor_id key')
        cls.flavor_id = CONF.magnum.flavor_id

    @classmethod
    def set_magnum_url(cls, config):
        cls.magnum_url = CONF.magnum.get('magnum_url', None)

    @classmethod
    def set_master_flavor_id(cls, config):
        if 'master_flavor_id' not in CONF.magnum:
            raise Exception('config missing master_flavor_id key')
        cls.master_flavor_id = CONF.magnum.master_flavor_id

    @classmethod
    def set_csr_location(cls, config):
        if 'csr_location' not in CONF.magnum:
            raise Exception('config missing csr_location key')
        cls.csr_location = CONF.magnum.csr_location

    @classmethod
    def set_dns_nameserver(cls, config):
        if 'dns_nameserver' not in CONF.magnum:
            raise Exception('config missing dns_nameserver')
        cls.dns_nameserver = CONF.magnum.dns_nameserver

    @classmethod
    def set_copy_logs(cls, config):
        if 'copy_logs' not in CONF.magnum:
            cls.copy_logs = True
        cls.copy_logs = str(CONF.magnum.copy_logs).lower() == 'true'

    @classmethod
    def setUp(cls):
        cls.set_admin_creds(config)
        cls.set_user_creds(config)
        cls.set_auth_version(config)
        cls.set_auth_url(config)
        cls.set_admin_role(config)

        cls.set_region(config)
        cls.set_image_id(config)
        cls.set_nic_id(config)
        cls.set_keypair_id(config)
        cls.set_flavor_id(config)
        cls.set_magnum_url(config)
        cls.set_master_flavor_id(config)
        cls.set_csr_location(config)
        cls.set_dns_nameserver(config)
        cls.set_copy_logs(config)



# Copyright (c) 2018 European Organization for Nuclear Research.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
from unittest import mock

from oslo_utils import timeutils
from oslo_utils import uuidutils

from magnum.api.controllers.v1 import nodegroup as api_nodegroup
from magnum.conductor import api as rpcapi
import magnum.conf
from magnum import objects
from magnum.tests import base
from magnum.tests.unit.api import base as api_base
from magnum.tests.unit.api import utils as apiutils
from magnum.tests.unit.db import utils as db_utils
from magnum.tests.unit.objects import utils as obj_utils

CONF = magnum.conf.CONF


class TestNodegroupObject(base.TestCase):
    def test_nodegroup_init(self):
        nodegroup_dict = apiutils.nodegroup_post_data()
        del nodegroup_dict['node_count']
        del nodegroup_dict['min_node_count']
        del nodegroup_dict['max_node_count']
        nodegroup = api_nodegroup.NodeGroup(**nodegroup_dict)
        self.assertEqual(1, nodegroup.node_count)
        self.assertEqual(0, nodegroup.min_node_count)
        self.assertIsNone(nodegroup.max_node_count)


class NodeGroupControllerTest(api_base.FunctionalTest):
    headers = {"Openstack-Api-Version": "container-infra latest"}

    def _add_headers(self, kwargs, roles=None):
        if 'headers' not in kwargs:
            kwargs['headers'] = self.headers
            if roles:
                kwargs['headers']['X-Roles'] = ",".join(roles)

    def get_json(self, *args, **kwargs):
        self._add_headers(kwargs, roles=['reader'])
        return super(NodeGroupControllerTest, self).get_json(*args, **kwargs)

    def post_json(self, *args, **kwargs):
        self._add_headers(kwargs, roles=['member'])
        return super(NodeGroupControllerTest, self).post_json(*args, **kwargs)

    def delete(self, *args, **kwargs):
        self._add_headers(kwargs, roles=['member'])
        return super(NodeGroupControllerTest, self).delete(*args, **kwargs)

    def patch_json(self, *args, **kwargs):
        self._add_headers(kwargs, roles=['member'])
        return super(NodeGroupControllerTest, self).patch_json(*args, **kwargs)


class TestListNodegroups(NodeGroupControllerTest):
    _expanded_attrs = ["id", "project_id", "docker_volume_size", "labels",
                       "node_addresses", "links"]

    _nodegroup_attrs = ["uuid", "name", "flavor_id", "node_count", "role",
                        "is_default", "image_id", "min_node_count",
                        "max_node_count"]

    def setUp(self):
        super(TestListNodegroups, self).setUp()
        obj_utils.create_test_cluster_template(self.context)
        self.cluster_uuid = uuidutils.generate_uuid()
        obj_utils.create_test_cluster(
            self.context, uuid=self.cluster_uuid)
        self.cluster = objects.Cluster.get_by_uuid(self.context,
                                                   self.cluster_uuid)

    def _test_list_nodegroups(self, cluster_id, filters=None, expected=None):
        url = '/clusters/%s/nodegroups' % cluster_id
        if filters is not None:
            filter_list = ['%s=%s' % (k, v) for k, v in filters.items()]
            url += '?' + '&'.join(f for f in filter_list)
        response = self.get_json(url)
        if expected is None:
            expected = []
        ng_uuids = [ng['uuid'] for ng in response['nodegroups']]
        self.assertEqual(expected, ng_uuids)
        for ng in response['nodegroups']:
            self._verify_attrs(self._nodegroup_attrs, ng)
            self._verify_attrs(self._expanded_attrs, ng, positive=False)

    def test_get_all(self):
        expected = [ng.uuid for ng in self.cluster.nodegroups]
        self._test_list_nodegroups(self.cluster_uuid, expected=expected)

    def test_get_all_by_name(self):
        expected = [ng.uuid for ng in self.cluster.nodegroups]
        self._test_list_nodegroups(self.cluster.name, expected=expected)

    def test_get_all_by_name_non_default_ngs(self):
        db_utils.create_test_nodegroup(cluster_id=self.cluster_uuid,
                                       name='non_default_ng')
        expected = [ng.uuid for ng in self.cluster.nodegroups]
        self._test_list_nodegroups(self.cluster.name, expected=expected)

    def test_get_all_with_pagination_marker(self):
        worker_ng_uuid = self.cluster.default_ng_worker.uuid
        master_ng_uuid = self.cluster.default_ng_master.uuid
        # First make sure that the api returns 1 ng and since they
        # are sorted by id, the ng should be the default-worker
        url = '/clusters/%s/nodegroups?limit=1' % (self.cluster_uuid)
        response = self.get_json(url)
        self.assertEqual(1, len(response['nodegroups']))
        self.assertEqual(worker_ng_uuid, response['nodegroups'][0]['uuid'])
        marker = "marker=%s" % worker_ng_uuid
        self.assertIn(marker, response['next'])
        # Now using the next url make sure that we get the default-master
        next_url = response['next'].split('v1')[1]
        response = self.get_json(next_url)
        self.assertEqual(1, len(response['nodegroups']))
        self.assertEqual(master_ng_uuid, response['nodegroups'][0]['uuid'])
        marker = "marker=%s" % master_ng_uuid
        self.assertIn(marker, response['next'])
        # Now we should not get any other entry since the cluster only has two
        # nodegroups and the marker is set at the default-master.
        next_url = response['next'].split('v1')[1]
        response = self.get_json(next_url)
        self.assertEqual(0, len(response['nodegroups']))
        self.assertNotIn('next', response)

    def test_get_all_by_role(self):
        filters = {'role': 'master'}
        expected = [self.cluster.default_ng_master.uuid]
        self._test_list_nodegroups(self.cluster.name, filters=filters,
                                   expected=expected)
        filters = {'role': 'worker'}
        expected = [self.cluster.default_ng_worker.uuid]
        self._test_list_nodegroups(self.cluster.name, filters=filters,
                                   expected=expected)

    def test_get_all_by_non_existent_role(self):
        filters = {'role': 'non-existent'}
        self._test_list_nodegroups(self.cluster.name, filters=filters)

    @mock.patch("magnum.common.policy.enforce")
    @mock.patch("magnum.common.context.make_context")
    def test_get_all_as_admin(self, mock_context, mock_policy):
        temp_uuid = uuidutils.generate_uuid()
        obj_utils.create_test_cluster(self.context, uuid=temp_uuid,
                                      project_id=temp_uuid)
        self.context.is_admin = True
        self.context.all_tenants = True
        cluster = objects.Cluster.get_by_uuid(self.context, temp_uuid)
        expected = [ng.uuid for ng in cluster.nodegroups]
        self._test_list_nodegroups(cluster.uuid, expected=expected)

    def test_get_all_non_existent_cluster(self):
        response = self.get_json('/clusters/not-here/nodegroups',
                                 expect_errors=True)
        self.assertEqual(404, response.status_code)

    def test_get_one(self):
        worker = self.cluster.default_ng_worker
        url = '/clusters/%s/nodegroups/%s' % (self.cluster.uuid, worker.uuid)
        response = self.get_json(url)
        self.assertEqual(worker.name, response['name'])
        self._verify_attrs(self._nodegroup_attrs, response)
        self._verify_attrs(self._expanded_attrs, response)
        self.assertEqual({}, response['labels_overridden'])
        self.assertEqual({}, response['labels_skipped'])
        self.assertEqual({}, response['labels_added'])

    def test_get_one_non_default(self):
        self.cluster.labels = {'label1': 'value1', 'label2': 'value2'}
        self.cluster.save()
        ng_name = 'non_default_ng'
        ng_labels = {
            'label1': 'value3', 'label2': 'value2', 'label4': 'value4'
        }
        db_utils.create_test_nodegroup(cluster_id=self.cluster.uuid,
                                       name=ng_name, labels=ng_labels)
        url = '/clusters/%s/nodegroups/%s' % (self.cluster.uuid, ng_name)
        response = self.get_json(url)
        self._verify_attrs(self._nodegroup_attrs, response)
        self._verify_attrs(self._expanded_attrs, response)
        self.assertEqual(ng_labels, response['labels'])
        overridden_labels = {'label1': 'value1'}
        self.assertEqual(overridden_labels, response['labels_overridden'])
        self.assertEqual({'label4': 'value4'}, response['labels_added'])
        self.assertEqual({}, response['labels_skipped'])

    def test_get_one_non_default_skipped_labels(self):
        self.cluster.labels = {'label1': 'value1', 'label2': 'value2'}
        self.cluster.save()
        ng_name = 'non_default_ng'
        ng_labels = {'label1': 'value3', 'label4': 'value4'}
        db_utils.create_test_nodegroup(cluster_id=self.cluster.uuid,
                                       name=ng_name, labels=ng_labels)
        url = '/clusters/%s/nodegroups/%s' % (self.cluster.uuid, ng_name)
        response = self.get_json(url)
        self._verify_attrs(self._nodegroup_attrs, response)
        self._verify_attrs(self._expanded_attrs, response)
        self.assertEqual(ng_labels, response['labels'])
        overridden_labels = {'label1': 'value1'}
        self.assertEqual(overridden_labels, response['labels_overridden'])
        self.assertEqual({'label4': 'value4'}, response['labels_added'])
        self.assertEqual({'label2': 'value2'}, response['labels_skipped'])

    def test_get_one_non_existent_ng(self):
        url = '/clusters/%s/nodegroups/not-here' % self.cluster.uuid
        response = self.get_json(url, expect_errors=True)
        self.assertEqual(404, response.status_code)

    @mock.patch("magnum.common.policy.enforce")
    @mock.patch("magnum.common.context.make_context")
    def test_get_one_as_admin(self, mock_context, mock_policy):
        temp_uuid = uuidutils.generate_uuid()
        obj_utils.create_test_cluster(self.context, uuid=temp_uuid,
                                      project_id=temp_uuid)
        self.context.is_admin = True
        self.context.all_tenants = True
        cluster = objects.Cluster.get_by_uuid(self.context, temp_uuid)
        worker = cluster.default_ng_worker
        url = '/clusters/%s/nodegroups/%s' % (cluster.uuid, worker.uuid)
        response = self.get_json(url)
        self.assertEqual(worker.name, response['name'])
        self._verify_attrs(self._nodegroup_attrs, response)
        self._verify_attrs(self._expanded_attrs, response)

    def test_get_one_wrong_microversion(self):
        headers = {"Openstack-Api-Version": "container-infra 1.8"}
        worker = self.cluster.default_ng_worker
        url = '/clusters/%s/nodegroups/%s' % (self.cluster.uuid, worker.uuid)
        response = self.get_json(url, headers=headers, expect_errors=True)
        self.assertEqual(406, response.status_code)

    def test_get_all_wrong_microversion(self):
        headers = {"Openstack-Api-Version": "container-infra 1.8"}
        url = '/clusters/%s/nodegroups/' % (self.cluster.uuid)
        response = self.get_json(url, headers=headers, expect_errors=True)
        self.assertEqual(406, response.status_code)


class TestPost(NodeGroupControllerTest):
    def setUp(self):
        super(TestPost, self).setUp()
        self.cluster_template = obj_utils.create_test_cluster_template(
            self.context)
        self.cluster = obj_utils.create_test_cluster(self.context)
        self.cluster.refresh()
        p = mock.patch.object(rpcapi.API, 'nodegroup_create_async')
        self.mock_ng_create = p.start()
        self.mock_ng_create.side_effect = self._simulate_nodegroup_create
        self.addCleanup(p.stop)
        self.url = "/clusters/%s/nodegroups" % self.cluster.uuid

    def _simulate_nodegroup_create(self, cluster, nodegroup):
        nodegroup.create()
        return nodegroup

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_nodegroup(self, mock_utcnow):
        ng_dict = apiutils.nodegroup_post_data()
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        self.assertTrue(uuidutils.is_uuid_like(response.json['uuid']))
        self.assertFalse(response.json['is_default'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_nodegroup_without_node_count(self, mock_utcnow):
        ng_dict = apiutils.nodegroup_post_data()
        del ng_dict['node_count']
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        # Verify node_count defaults to 1
        self.assertEqual(1, response.json['node_count'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_nodegroup_with_zero_nodes(self, mock_utcnow):
        ng_dict = apiutils.nodegroup_post_data()
        ng_dict['node_count'] = 0
        ng_dict['min_node_count'] = 0
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        # Verify node_count is set to zero
        self.assertEqual(0, response.json['node_count'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_nodegroup_with_max_node_count(self, mock_utcnow):
        ng_dict = apiutils.nodegroup_post_data(max_node_count=5)
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        self.assertEqual(5, response.json['max_node_count'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_nodegroup_with_role(self, mock_utcnow):
        ng_dict = apiutils.nodegroup_post_data(role='test-role')
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        self.assertEqual('test-role', response.json['role'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_nodegroup_with_labels(self, mock_utcnow):
        labels = {'label1': 'value1'}
        ng_dict = apiutils.nodegroup_post_data(labels=labels)
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        self.assertEqual(labels, response.json['labels'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_nodegroup_with_image_id(self, mock_utcnow):
        ng_dict = apiutils.nodegroup_post_data(image_id='test_image')
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        self.assertEqual('test_image', response.json['image_id'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_nodegroup_with_flavor(self, mock_utcnow):
        ng_dict = apiutils.nodegroup_post_data(flavor_id='test_flavor')
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        self.assertEqual('test_flavor', response.json['flavor_id'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_nodegroup_only_name(self, mock_utcnow):
        ng_dict = {'name': 'test_ng'}
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        self.assertEqual('worker', response.json['role'])
        self.assertEqual(self.cluster_template.image_id,
                         response.json['image_id'])
        self.assertEqual(self.cluster.flavor_id, response.json['flavor_id'])
        self.assertEqual(self.cluster.uuid, response.json['cluster_id'])
        self.assertEqual(self.cluster.project_id, response.json['project_id'])
        self.assertEqual(self.cluster.labels, response.json['labels'])
        self.assertEqual('worker', response.json['role'])
        self.assertEqual(0, response.json['min_node_count'])
        self.assertEqual(1, response.json['node_count'])
        self.assertIsNone(response.json['max_node_count'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_nodegroup_invalid_node_count(self, mock_utcnow):
        ng_dict = apiutils.nodegroup_post_data(node_count=7, max_node_count=5)
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.post_json(self.url, ng_dict, expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(409, response.status_int)

        ng_dict = apiutils.nodegroup_post_data(node_count=2, min_node_count=3)

        response = self.post_json(self.url, ng_dict, expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(409, response.status_int)

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_master_ng(self, mock_utcnow):
        ng_dict = apiutils.nodegroup_post_data(role='master')
        response = self.post_json(self.url, ng_dict, expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(400, response.status_int)

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_ng_same_name(self, mock_utcnow):
        existing_name = self.cluster.default_ng_master.name
        ng_dict = apiutils.nodegroup_post_data(name=existing_name)
        response = self.post_json(self.url, ng_dict, expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(409, response.status_int)

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_create_ng_wrong_microversion(self, mock_utcnow):
        headers = {"Openstack-Api-Version": "container-infra 1.8"}
        ng_dict = apiutils.nodegroup_post_data(name="new_ng")
        response = self.post_json(self.url, ng_dict, headers=headers,
                                  expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(406, response.status_int)

    def test_create_ng_cluster_no_api_address(self):
        # Remove the api address from the cluster and make sure
        # that the request is not accepted.
        self.cluster.api_address = None
        self.cluster.save()
        ng_dict = apiutils.nodegroup_post_data()
        response = self.post_json(self.url, ng_dict, expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(409, response.status_int)

    def test_create_ng_with_labels(self):
        cluster_labels = {'label1': 'value1', 'label2': 'value2'}
        self.cluster.labels = cluster_labels
        self.cluster.save()
        ng_labels = {'label3': 'value3'}
        ng_dict = apiutils.nodegroup_post_data(labels=ng_labels)
        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        (cluster, ng), _ = self.mock_ng_create.call_args
        self.assertEqual(ng_labels, ng.labels)

    def test_create_ng_with_merge_labels(self):
        cluster_labels = {'label1': 'value1', 'label2': 'value2'}
        self.cluster.labels = cluster_labels
        self.cluster.save()
        ng_labels = {'label1': 'value3', 'label4': 'value4'}
        ng_dict = apiutils.nodegroup_post_data(labels=ng_labels,
                                               merge_labels=True)
        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        (cluster, ng), _ = self.mock_ng_create.call_args
        expected_labels = cluster.labels
        expected_labels.update(ng_labels)
        self.assertEqual(expected_labels, ng.labels)

    def test_create_ng_with_merge_labels_no_labels(self):
        cluster_labels = {'label1': 'value1', 'label2': 'value2'}
        self.cluster.labels = cluster_labels
        self.cluster.save()
        ng_dict = apiutils.nodegroup_post_data(merge_labels=True)
        ng_dict.pop('labels')
        response = self.post_json(self.url, ng_dict)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_int)
        (cluster, ng), _ = self.mock_ng_create.call_args
        self.assertEqual(cluster.labels, ng.labels)


class TestDelete(NodeGroupControllerTest):

    def setUp(self):
        super(TestDelete, self).setUp()
        self.cluster_template = obj_utils.create_test_cluster_template(
            self.context)
        self.cluster = obj_utils.create_test_cluster(self.context)
        self.cluster.refresh()
        self.nodegroup = obj_utils.create_test_nodegroup(
            self.context, cluster_id=self.cluster.uuid, is_default=False)
        p = mock.patch.object(rpcapi.API, 'nodegroup_delete_async')
        self.mock_ng_delete = p.start()
        self.mock_ng_delete.side_effect = self._simulate_nodegroup_delete
        self.addCleanup(p.stop)
        self.url = "/clusters/%s/nodegroups/" % self.cluster.uuid

    def _simulate_nodegroup_delete(self, cluster, nodegroup):
        nodegroup.destroy()

    def test_delete_nodegroup(self):
        response = self.delete(self.url + self.nodegroup.uuid)
        self.assertEqual(204, response.status_int)
        response = self.get_json(self.url + self.nodegroup.uuid,
                                 expect_errors=True)
        self.assertEqual(404, response.status_int)
        self.assertEqual('application/json', response.content_type)
        self.assertIsNotNone(response.json['errors'])

    def test_delete_nodegroup_by_name(self):
        response = self.delete(self.url + self.nodegroup.name)
        self.assertEqual(204, response.status_int)
        response = self.get_json(self.url + self.nodegroup.name,
                                 expect_errors=True)
        self.assertEqual(404, response.status_int)
        self.assertEqual('application/json', response.content_type)
        self.assertIsNotNone(response.json['errors'])

    def test_delete_not_found(self):
        uuid = uuidutils.generate_uuid()
        response = self.delete(self.url + uuid, expect_errors=True)
        self.assertEqual(404, response.status_int)
        self.assertEqual('application/json', response.content_type)
        self.assertIsNotNone(response.json['errors'])

    def test_delete_by_name_not_found(self):
        response = self.delete(self.url + "not-there", expect_errors=True)
        self.assertEqual(404, response.status_int)
        self.assertEqual('application/json', response.content_type)
        self.assertIsNotNone(response.json['errors'])

    def test_delete_default_nodegroup(self):
        response = self.delete(self.url + self.cluster.default_ng_master.uuid,
                               expect_errors=True)
        self.assertEqual(400, response.status_int)
        self.assertEqual('application/json', response.content_type)
        self.assertIsNotNone(response.json['errors'])

    @mock.patch("magnum.common.policy.enforce")
    @mock.patch("magnum.common.context.make_context")
    def test_delete_nodegroup_as_admin(self, mock_context, mock_policy):
        cluster_uuid = uuidutils.generate_uuid()
        obj_utils.create_test_cluster(self.context, uuid=cluster_uuid,
                                      project_id='fake', name='test-fake')
        ng_uuid = uuidutils.generate_uuid()
        obj_utils.create_test_nodegroup(self.context, uuid=ng_uuid,
                                        cluster_id=cluster_uuid,
                                        is_default=False,
                                        project_id='fake', id=50)
        self.context.is_admin = True
        url = '/clusters/%s/nodegroups/%s' % (cluster_uuid, ng_uuid)
        response = self.delete(url)
        self.assertEqual(204, response.status_int)

    def test_delete_wrong_microversion(self):
        headers = {"Openstack-Api-Version": "container-infra 1.8"}
        response = self.delete(self.url + self.nodegroup.uuid, headers=headers,
                               expect_errors=True)
        self.assertEqual(406, response.status_int)


class TestPatch(NodeGroupControllerTest):
    def setUp(self):
        super(TestPatch, self).setUp()
        self.cluster_template = obj_utils.create_test_cluster_template(
            self.context)
        self.cluster = obj_utils.create_test_cluster(self.context)
        self.cluster.refresh()
        self.nodegroup = obj_utils.create_test_nodegroup(
            self.context, cluster_id=self.cluster.uuid, is_default=False,
            min_node_count=2, max_node_count=5, node_count=2)
        p = mock.patch.object(rpcapi.API, 'nodegroup_update_async')
        self.mock_ng_update = p.start()
        self.mock_ng_update.side_effect = self._simulate_nodegroup_update
        self.addCleanup(p.stop)
        self.url = "/clusters/%s/nodegroups/" % self.cluster.uuid

    def _simulate_nodegroup_update(self, cluster, nodegroup):
        nodegroup.save()
        return nodegroup

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_replace_ok(self, mock_utcnow):
        max_node_count = 4
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.patch_json(self.url + self.nodegroup.uuid,
                                   [{'path': '/max_node_count',
                                     'value': max_node_count,
                                     'op': 'replace'}])
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_code)

        response = self.get_json(self.url + self.nodegroup.uuid)
        self.assertEqual(max_node_count, response['max_node_count'])
        return_updated_at = timeutils.parse_isotime(
            response['updated_at']).replace(tzinfo=None)
        self.assertEqual(test_time, return_updated_at)

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_replace_ok_by_name(self, mock_utcnow):
        max_node_count = 4
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.patch_json(self.url + self.nodegroup.name,
                                   [{'path': '/max_node_count',
                                     'value': max_node_count,
                                     'op': 'replace'}])
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_code)

        response = self.get_json(self.url + self.nodegroup.uuid)
        self.assertEqual(max_node_count, response['max_node_count'])
        return_updated_at = timeutils.parse_isotime(
            response['updated_at']).replace(tzinfo=None)
        self.assertEqual(test_time, return_updated_at)

    def test_replace_node_count_failed(self):
        response = self.patch_json(self.url + self.nodegroup.name,
                                   [{'path': '/node_count',
                                     'value': 3,
                                     'op': 'replace'}],
                                   expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(400, response.status_code)
        self.assertIsNotNone(response.json['errors'])

    def test_replace_max_node_count_failed(self):
        # min_node_count equals to 2. Verify that if the max_node_count
        # is less than the min the patch fails
        response = self.patch_json(self.url + self.nodegroup.name,
                                   [{'path': '/max_node_count',
                                     'value': 1,
                                     'op': 'replace'}],
                                   expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(409, response.status_code)
        self.assertIsNotNone(response.json['errors'])

    def test_replace_min_node_count_failed(self):
        # min_node_count equals to 2. Verify that if the max_node_count
        # is less than the min the patch fails
        response = self.patch_json(self.url + self.nodegroup.name,
                                   [{'path': '/min_node_count',
                                     'value': 3,
                                     'op': 'replace'}],
                                   expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(409, response.status_code)
        self.assertIsNotNone(response.json['errors'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_remove_ok(self, mock_utcnow):
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.patch_json(self.url + self.nodegroup.name,
                                   [{'path': '/max_node_count',
                                     'op': 'remove'}])
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_code)

        response = self.get_json(self.url + self.nodegroup.uuid)
        self.assertIsNone(response['max_node_count'])
        return_updated_at = timeutils.parse_isotime(
            response['updated_at']).replace(tzinfo=None)
        self.assertEqual(test_time, return_updated_at)

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_remove_min_node_count(self, mock_utcnow):
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.patch_json(self.url + self.nodegroup.name,
                                   [{'path': '/min_node_count',
                                     'op': 'remove'}])
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_code)

        response = self.get_json(self.url + self.nodegroup.uuid)
        # Removing the min_node_count just restores the default value
        self.assertEqual(0, response['min_node_count'])
        return_updated_at = timeutils.parse_isotime(
            response['updated_at']).replace(tzinfo=None)
        self.assertEqual(test_time, return_updated_at)

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_remove_internal_attr(self, mock_utcnow):
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.patch_json(self.url + self.nodegroup.name,
                                   [{'path': '/node_count',
                                     'op': 'remove'}], expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(400, response.status_code)
        self.assertIsNotNone(response.json['errors'])

    @mock.patch('oslo_utils.timeutils.utcnow')
    def test_remove_non_existent_property(self, mock_utcnow):
        test_time = datetime.datetime(2000, 1, 1, 0, 0)
        mock_utcnow.return_value = test_time

        response = self.patch_json(self.url + self.nodegroup.name,
                                   [{'path': '/not_there',
                                     'op': 'remove'}], expect_errors=True)
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(400, response.status_code)
        self.assertIsNotNone(response.json['errors'])

    @mock.patch("magnum.common.policy.enforce")
    @mock.patch("magnum.common.context.make_context")
    def test_update_nodegroup_as_admin(self, mock_context, mock_policy):
        cluster_uuid = uuidutils.generate_uuid()
        obj_utils.create_test_cluster(self.context, uuid=cluster_uuid,
                                      project_id='fake', name='test-fake')
        ng_uuid = uuidutils.generate_uuid()
        obj_utils.create_test_nodegroup(self.context, uuid=ng_uuid,
                                        cluster_id=cluster_uuid,
                                        is_default=False,
                                        project_id='fake', id=50)
        self.context.is_admin = True
        url = '/clusters/%s/nodegroups/%s' % (cluster_uuid, ng_uuid)
        response = self.patch_json(url,
                                   [{'path': '/max_node_count',
                                     'value': 4,
                                     'op': 'replace'}])
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(202, response.status_code)

    def test_replace_wrong_microversion(self):
        headers = {"Openstack-Api-Version": "container-infra 1.8"}
        response = self.patch_json(self.url + self.nodegroup.name,
                                   [{'path': '/max_node_count',
                                     'value': 4,
                                     'op': 'replace'}], headers=headers,
                                   expect_errors=True)
        self.assertEqual(406, response.status_code)


class TestNodeGroupPolicyEnforcement(NodeGroupControllerTest):
    def setUp(self):
        super(TestNodeGroupPolicyEnforcement, self).setUp()
        obj_utils.create_test_cluster_template(self.context)
        self.cluster_uuid = uuidutils.generate_uuid()
        obj_utils.create_test_cluster(
            self.context, uuid=self.cluster_uuid)
        self.cluster = objects.Cluster.get_by_uuid(self.context,
                                                   self.cluster_uuid)

    def _common_policy_check(self, rule, func, *arg, **kwarg):
        self.policy.set_rules({rule: "project:non_fake"})
        response = func(*arg, **kwarg)
        self.assertEqual(403, response.status_int)
        self.assertEqual('application/json', response.content_type)
        self.assertTrue(
            "Policy doesn't allow %s to be performed." % rule,
            response.json['errors'][0]['detail'])

    def test_policy_disallow_get_all(self):
        self._common_policy_check(
            "nodegroup:get_all", self.get_json,
            '/clusters/%s/nodegroups' % self.cluster_uuid, expect_errors=True)

    def test_policy_disallow_get_one(self):
        worker = self.cluster.default_ng_worker
        self._common_policy_check(
            "nodegroup:get", self.get_json,
            '/clusters/%s/nodegroups/%s' % (self.cluster.uuid, worker.uuid),
            expect_errors=True)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils
from webob import exc


class HTTPNotAcceptableAPIVersion(exc.HTTPNotAcceptable):
    # subclass of :class:`~HTTPNotAcceptable`
    #
    # This indicates the resource identified by the request is only
    # capable of generating response entities which have content
    # characteristics not acceptable according to the accept headers
    # sent in the request.
    #
    # code: 406, title: Not Acceptable
    #
    # differences from webob.exc.HTTPNotAcceptable:
    #
    # - additional max and min version parameters
    # - additional error info for code, title, and links
    code = 406
    title = 'Not Acceptable'
    max_version = ''
    min_version = ''

    def __init__(self, detail=None, headers=None, comment=None,
                 body_template=None, max_version='', min_version='', **kw):

        super(HTTPNotAcceptableAPIVersion, self).__init__(
            detail=detail, headers=headers, comment=comment,
            body_template=body_template, **kw)

        self.max_version = max_version
        self.min_version = min_version

    def __call__(self, environ, start_response):
        for err_str in self.app_iter:
            err = {}
            try:
                err = jsonutils.loads(err_str.decode('utf-8'))
            except ValueError:
                pass

            links = {'rel': 'help', 'href': 'http://docs.openstack.org'
                     '/api-guide/compute/microversions.html'}

            err['max_version'] = self.max_version
            err['min_version'] = self.min_version
            err['code'] = "magnum.microversion-unsupported"
            err['links'] = [links]
            err['title'] = "Requested microversion is unsupported"

        self.app_iter = [jsonutils.dump_as_bytes(err)]
        self.headers['Content-Length'] = str(len(self.app_iter[0]))

        return super(HTTPNotAcceptableAPIVersion, self).__call__(
            environ, start_response)
